//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module BaseExtender #(
  )(
  input logic [1:0] inMod3,
  input logic [1:0] inMod4,
  input logic [2:0] inMod5,
  input logic [2:0] inMod7,
  output logic outIsNeg,
  output logic [3:0] outMod11
  );

  // mod4 is m_3, mod3 is m_2, mod5 is m_1 and mod7 is m_0
  logic[1:0] aMod3;
  logic[1:0] aMod4;
  logic[2:0] aMod5;

  logic[1:0] r2Mod4;
  logic[1:0] r1Mod4;
  logic[1:0] r1Mod3;

  logic[1:0] r1Mod3Rom[15:0];
  genvar i;
  generate
  for(i = 0;i < 16; i++) begin: la_r1Mod3
    if (i < 3)
      assign r1Mod3Rom[i] = i;
    else
      assign r1Mod3Rom[i] = (i-1)%3;
  end: la_r1Mod3
  endgenerate
   
  logic[1:0] r1Mod4Rom[3:0];
  generate
  for(i = 0;i < 4 ; i++) begin: la_r1Mod4
    if (i < 4)
      assign r1Mod4Rom[i] = i*3;
  end: la_r1Mod4
  endgenerate
  
  logic[1:0] r2Mod4Rom[3:0];
  generate
  for(i = 0;i < 4 ; i++) begin: la_r2Mod4
    if (i < 4)
      assign r2Mod4Rom[i] = i;
  end: la_r2Mod4
  endgenerate


  logic[2:0] aMod5Rom[15:0];
  generate
  for(i = 0;i < 16; i++) begin: la_aMod5
    if (i < 5)
      assign aMod5Rom[i] = (i*3)%5;
    else
      assign aMod5Rom[i] = ((i-1)%5*3)%5;
  end: la_aMod5
  endgenerate

  logic[1:0] aMod3Rom[7:0];
  generate
  for(i = 0;i < 8; i++) begin: la_aMod3
    if (i < 3)
      assign aMod3Rom[i] = (i*2)%3;
    else
      assign aMod3Rom[i] = ((i-2)%3*2)%3;
  end: la_aMod3
  endgenerate

  logic[1:0] aMod4Rom[3:0];
  generate
  for(i = 0;i < 4 ; i++) begin: la_aMod4
    if (i < 4)
      assign aMod4Rom[i] = i*3;
  end: la_aMod4
  endgenerate
  

  logic[2:0] aMod3subs;
  assign aMod3subs = r1Mod3 - aMod5;
  assign aMod3 = aMod3Rom[aMod3subs];

  logic[1:0] aMod4subs;
  assign aMod4subs = r2Mod4 - aMod3;
  assign aMod4 = aMod4Rom[aMod4subs];

  logic[3:0] aMod5subs;
  assign aMod5subs = inMod5 - inMod7;
  assign aMod5 = aMod5Rom[aMod5subs];

  logic[3:0] r1Mod3subs;
  assign r1Mod3subs = inMod3 - inMod7;
  assign r1Mod3 = r1Mod3Rom[r1Mod3subs];

 
  logic[1:0] r1Mod4subs;
  assign r1Mod4subs = inMod4 - inMod7;
  assign r1Mod4 = r1Mod4Rom[r1Mod4subs];

  logic[1:0] r2Mod4subs;
  assign r2Mod4subs = r1Mod4 - aMod5;
  assign r2Mod4 = r2Mod4Rom[r2Mod4subs];


  logic[3:0] multMod3, multMod4, multMod5;
  
  logic[3:0] multMod3ROM[2:0];
  generate
  for(i = 0; i < 3; i++) begin:la_multMod3ROM
    assign multMod3ROM[i] = (i*5*7)%11;
  end:la_multMod3ROM
  endgenerate

  logic[3:0] multMod4ROM[3:0];
  generate
  for(i = 0; i < 4; i++) begin:la_multMod4ROM
    assign multMod4ROM[i] = (i*3*5*7)%11;
  end:la_multMod4ROM
  endgenerate
 
  logic[3:0] multMod5ROM[4:0];
  generate
  for(i = 0; i < 5; i++) begin:la_multMod5ROM
    assign multMod5ROM[i] = (i*7)%11;
  end:la_multMod5ROM
  endgenerate
 
  assign multMod3 = multMod3ROM[aMod3];
  assign multMod4 = multMod4ROM[aMod4];
  assign multMod5 = multMod5ROM[aMod5];

  logic[4:0] baseSum11, baseSum12, baseSum2;
  logic[3:0] baseSum11corr, baseSum12corr, baseSum2corr;

  assign baseSum11 = multMod5 + multMod4;
  assign baseSum12 = multMod3 + inMod7;
  assign baseSum11corr = baseSum11 < 11? baseSum11 : baseSum11 -11; 
  assign baseSum12corr = baseSum12 < 11? baseSum12 : baseSum12 -11;
  assign baseSum2 = baseSum11corr + baseSum12corr;
  assign baseSum2corr = baseSum2 < 11? baseSum2 : baseSum2 -11;
  
  assign outIsNeg = aMod4 >= 2;
  assign outMod11 = baseSum2corr;
  
endmodule
