//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module HybridFirLinear #(
  parameter NUMBER_OF_MODULI								= pa_Receiver12bits::FIR_NUMBER_OF_MODULI,
  //parameter integer MODULI[NUMBER_OF_MODULI -1 :0]					= pa_Receiver16bits::FIR_MODULI,
  parameter NUMBER_OF_TAPS								= pa_Receiver12bits::FIR_NUMBER_OF_TAPS
//  parameter integer COEFFICIENTS[NUMBER_OF_MODULI - 1:0][NUMBER_OF_TAPS - 1:0]		= pa_Receiver16bits::FIR_COEFFICIENTS
  )(
  input logic ck,
  input logic [1:0] iMod3I,
  input logic [1:0] iMod4I,
  input logic [2:0] iMod5I,
  input logic [2:0] iMod7I,
  input logic [3:0] iMod11I,
  input logic [1:0] coeffMod3 [NUMBER_OF_TAPS -1:0],
  input logic [1:0] coeffMod4 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod5 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod7 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod11 [NUMBER_OF_TAPS -1:0],
  output logic [1:0] oMod3I,
  output logic [1:0] oMod4I,
  output logic [2:0] oMod5I,
  output logic [2:0] oMod7I,
  output logic [3:0] oMod11I
  );

// MODULO 3 FIR filter
 
logic [1:0] xMod3I[NUMBER_OF_TAPS:0];
logic [1:0] xMod3Inext[NUMBER_OF_TAPS:0];
logic [1:0] xMod3Ireg[NUMBER_OF_TAPS:0];
logic [4:0] yMod3I[NUMBER_OF_TAPS:0];
logic [4:0] yOutRegMod3;

genvar i;
generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod3
    HybridFirCellLinear#(.WORD_LENGTH(2),.MODULO(3)) u_HybridCellMod3(xMod3I[i],yMod3I[i],coeffMod3[i],xMod3Ireg[i+1],yMod3I[i+1]);
    assign xMod3I[i+1] = xMod3Inext[i+1];
end: la_firMod3
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod3
    xMod3Inext[i] <= xMod3Ireg[i];
  end: la_regMod3
  yOutRegMod3 = yMod3I[NUMBER_OF_TAPS];
end

assign xMod3I[0] = iMod3I;
assign yMod3I[0] = 5'b00000;

InputConverter #(.WORD_LENGTH(6),.MODULO(3),.MODULO_WORD_LENGTH(2)) u_ResidueConverterMod3I({1'b0,yOutRegMod3},oMod3I);

// MODULO 4 FIR filter

logic [NUMBER_OF_TAPS:0][1:0] xMod4I;
logic [NUMBER_OF_TAPS:0][1:0] xMod4Inext;
logic [NUMBER_OF_TAPS:0][1:0] xMod4Iprod;
logic [NUMBER_OF_TAPS:0][1:0] xMod4Ireg;
logic [NUMBER_OF_TAPS:0][1:0] yMod4I;
logic [1:0] yOutRegMod4;


generate
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod4
    assign xMod4Iprod[i] = xMod4I[i]*coeffMod4[i];
    assign xMod4Ireg[i+1] = xMod4I[i];
    assign yMod4I[i+1] = yMod4I[i] + xMod4Iprod[i];
    assign xMod4I[i+1] = xMod4Inext[i+1];
  end: la_firMod4
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod4
    xMod4Inext[i] <= xMod4Ireg[i];
  end: la_regMod4
  yOutRegMod4 <= yMod4I[NUMBER_OF_TAPS];
end


assign xMod4I[0] = iMod4I;
assign yMod4I[0] = 3'b000;


assign oMod4I  = yOutRegMod4;

// MODULO 5 FIR filter 

logic [NUMBER_OF_TAPS:0][2:0] xMod5I;
logic [2:0] xMod5Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][2:0] xMod5Ireg;
logic [NUMBER_OF_TAPS:0][6:0] yMod5I;
logic [6:0] yOutRegMod5;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod5
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(5)) u_HybridCellMod5(xMod5I[i],yMod5I[i],coeffMod5[i],xMod5Ireg[i+1],yMod5I[i+1]);
    assign xMod5I[i+1] = xMod5Inext[i+1];
end: la_firMod5
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod5
    xMod5Inext[i] <= xMod5Ireg[i];
  end: la_regMod5
  yOutRegMod5 <= yMod5I[NUMBER_OF_TAPS];
end

assign xMod5I[0] = iMod5I;
assign yMod5I[0] = 7'b0000000;



InputConverter #(.WORD_LENGTH(8),.MODULO(5),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod5I({1'b0,yOutRegMod5},oMod5I);

// MODULO 7 FIR filter 

logic [NUMBER_OF_TAPS:0][2:0] xMod7I;
logic [NUMBER_OF_TAPS:0][2:0] xMod7Ireg;
logic [2:0] xMod7Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][6:0] yMod7I;
logic [6:0] yOutRegMod7;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod7
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(7)) u_HybridCellMod7(xMod7I[i],yMod7I[i],coeffMod7[i],xMod7Ireg[i+1],yMod7I[i+1]);
    assign xMod7I[i+1] = xMod7Inext[i+1];
end: la_firMod7
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod7
    xMod7Inext[i] <= xMod7Ireg[i];
  end: la_regMod7
  yOutRegMod7 <= yMod7I[NUMBER_OF_TAPS];
end

assign xMod7I[0] = iMod7I;
assign yMod7I[0] = 7'b0000000;
  
InputConverter #(.WORD_LENGTH(8),.MODULO(7),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod7I({1'b0,yOutRegMod7},oMod7I);

// MODULO 11 FIR filter 

logic [NUMBER_OF_TAPS:0][3:0] xMod11I;
logic [3:0] xMod11Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][3:0] xMod11Ireg;
logic [NUMBER_OF_TAPS:0][8:0] yMod11I;
logic [8:0] yOutRegMod11;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod11
    HybridFirCellLinear#(.WORD_LENGTH(4),.MODULO(11)) u_HybridCellMod11(xMod11I[i],yMod11I[i],coeffMod11[i],xMod11Ireg[i+1],yMod11I[i+1]);
    assign xMod11I[i+1] = xMod11Inext[i+1];
end: la_firMod11
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod11
    xMod11Inext[i] <= xMod11Ireg[i];
  end: la_regMod11
  yOutRegMod11 <= yMod11I[NUMBER_OF_TAPS];
end

assign xMod11I[0] = iMod11I;
assign yMod11I[0] = 9'b000000000;

InputConverter #(.WORD_LENGTH(10),.MODULO(11),.MODULO_WORD_LENGTH(4)) u_ResidueConverterMod11I({1'b0,yOutRegMod11},oMod11I);
endmodule
