//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


  `include "NumberOfTaps.sv"
module Receiver12bits #( 
  parameter INCLUDE_RECEIVER16BITS    = pa_Receiver12bits::INCLUDE_RECEIVER16BITS,
  parameter NUMBER_OF_MODULI	      = pa_Receiver12bits::FIR_NUMBER_OF_MODULI,
  parameter integer MODULI [NUMBER_OF_MODULI - 1:0]      = pa_Receiver12bits::FIR_MODULI 
  )(
  input logic [11:0] signalInI,
  input logic [11:0] signalInQ, 
  input logic [1:0] coeffMod3I [NUMBER_OF_TAPS -1:0],
  input logic [1:0] coeffMod4I [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod5I [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod7I [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod11I [NUMBER_OF_TAPS -1:0],
  input ck, 
  output logic[11:0] converterOutI,
  output logic[11:0] converterOutQ
  );
  


  logic [1:0] convOutMod3I, convOutMod3Q; 
  logic [1:0] convOutMod4I, convOutMod4Q;
  logic [2:0] convOutMod5I, convOutMod5Q;
  logic [2:0] convOutMod7I, convOutMod7Q;
  logic [3:0] convOutMod11I, convOutMod11Q;

  logic [1:0] convOutMod3Ireg, convOutMod3Qreg; 
  logic [1:0] convOutMod4Ireg, convOutMod4Qreg;
  logic [2:0] convOutMod5Ireg, convOutMod5Qreg;
  logic [2:0] convOutMod7Ireg, convOutMod7Qreg;
  logic [3:0] convOutMod11Ireg, convOutMod11Qreg;
  

  logic [1:0] scalerOutMod3I, scalerOutMod3Q; 
  logic [1:0] scalerOutMod4I, scalerOutMod4Q;
  logic [2:0] scalerOutMod5I, scalerOutMod5Q;
  logic [2:0] scalerOutMod7I, scalerOutMod7Q;
  logic [3:0] scalerOutMod11I, scalerOutMod11Q;

  logic [1:0] scalerOutMod3Ireg, scalerOutMod3Qreg; 
  logic [1:0] scalerOutMod4Ireg, scalerOutMod4Qreg;
  logic [2:0] scalerOutMod5Ireg, scalerOutMod5Qreg;
  logic [2:0] scalerOutMod7Ireg, scalerOutMod7Qreg;
  logic [3:0] scalerOutMod11Ireg, scalerOutMod11Qreg;

  logic [1:0] filterOutMod3I, filterOutMod3Q; 
  logic [1:0] filterOutMod4I, filterOutMod4Q;
  logic [2:0] filterOutMod5I, filterOutMod5Q;
  logic [2:0] filterOutMod7I, filterOutMod7Q;
  logic [3:0] filterOutMod11I, filterOutMod11Q;

  logic [1:0] filterOutMod3Ireg, filterOutMod3Qreg; 
  logic [1:0] filterOutMod4Ireg, filterOutMod4Qreg;
  logic [2:0] filterOutMod5Ireg, filterOutMod5Qreg;
  logic [2:0] filterOutMod7Ireg, filterOutMod7Qreg;
  logic [3:0] filterOutMod11Ireg, filterOutMod11Qreg;
// Input Converters
 
InputConverter #(.WORD_LENGTH(12),
  .MODULO(3),
  .MODULO_WORD_LENGTH(2)
  ) u_InputConverterMod3I (
    .in (signalInI),
    .out (convOutMod3I)
    );
assign convOutMod4I = signalInI[1:0];
InputConverter #(.WORD_LENGTH(12),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5I (
    .in (signalInI),
    .out (convOutMod5I)
    );
InputConverter #(.WORD_LENGTH(12),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7I (
    .in (signalInI),
    .out (convOutMod7I)
    );
InputConverter #(.WORD_LENGTH(12),
  .MODULO(11),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod11I (
    .in (signalInI),
    .out (convOutMod11I)
    );

InputConverter #(.WORD_LENGTH(12),
  .MODULO(3),
  .MODULO_WORD_LENGTH(2)
  ) u_InputConverterMod3Q (
    .in (signalInQ),
    .out (convOutMod3Q)
    );
assign convOutMod4Q = signalInQ[2:0];
InputConverter #(.WORD_LENGTH(12),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5Q (
    .in (signalInQ),
    .out (convOutMod5Q)
    );
InputConverter #(.WORD_LENGTH(12),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7Q (
    .in (signalInQ),
    .out (convOutMod7Q)
    );
InputConverter #(.WORD_LENGTH(12),
  .MODULO(11),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod11Q (
    .in (signalInQ),
    .out (convOutMod11Q)
    );
always @(posedge ck) begin
  convOutMod3Ireg <= convOutMod3I;
  convOutMod3Qreg <= convOutMod3Q;
  convOutMod4Ireg <= convOutMod4I;
  convOutMod4Qreg <= convOutMod4Q;
  convOutMod5Ireg <= convOutMod5I;
  convOutMod5Qreg <= convOutMod5Q;
  convOutMod7Ireg <= convOutMod7I;
  convOutMod7Qreg <= convOutMod7Q;
  convOutMod11Ireg <= convOutMod11I;
  convOutMod11Qreg <= convOutMod11Q;
end
// FIR Filter  
  
HybridFirLinear #(.NUMBER_OF_MODULI(5), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearI(.ck(ck), .iMod3I(convOutMod3Ireg), .iMod4I(convOutMod4Ireg), .iMod5I(convOutMod5Ireg), .iMod7I(convOutMod7Ireg), .iMod11I(convOutMod11Ireg), .coeffMod3(coeffMod3I), .coeffMod4(coeffMod4I), .coeffMod5(coeffMod5I), .coeffMod7(coeffMod7I), .coeffMod11(coeffMod11I), .oMod3I(filterOutMod3I), .oMod4I(filterOutMod4I), .oMod5I(filterOutMod5I), .oMod7I(filterOutMod7I), .oMod11I(filterOutMod11I));


HybridFirLinear #(.NUMBER_OF_MODULI(5), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearQ(.ck(ck), .iMod3I(convOutMod3Qreg), .iMod4I(convOutMod4Qreg), .iMod5I(convOutMod5Qreg), .iMod7I(convOutMod7Qreg), .iMod11I(convOutMod11Qreg), .coeffMod3(coeffMod3I), .coeffMod4(coeffMod4I), .coeffMod5(coeffMod5I), .coeffMod7(coeffMod7I), .coeffMod11(coeffMod11I),  .oMod3I(filterOutMod3Q), .oMod4I(filterOutMod4Q), .oMod5I(filterOutMod5Q), .oMod7I(filterOutMod7Q), .oMod11I(filterOutMod11Q));

// Registers

always @(posedge ck) begin
  filterOutMod3Ireg <= filterOutMod3I;
  filterOutMod3Qreg <= filterOutMod3Q;
  filterOutMod4Ireg <= filterOutMod4I;
  filterOutMod4Qreg <= filterOutMod4Q;
  filterOutMod5Ireg <= filterOutMod5I;
  filterOutMod5Qreg <= filterOutMod5Q;
  filterOutMod7Ireg <= filterOutMod7I;
  filterOutMod7Qreg <= filterOutMod7Q;
  filterOutMod11Ireg <= filterOutMod11I;
  filterOutMod11Qreg <= filterOutMod11Q;
end

Scaler u_ScalerI(filterOutMod3Ireg,filterOutMod4Ireg,filterOutMod5Ireg,filterOutMod7Ireg,filterOutMod11Ireg,scalerOutMod3I,scalerOutMod4I,scalerOutMod5I,scalerOutMod7I,scalerOutMod11I);
Scaler u_ScalerQ(filterOutMod3Qreg,filterOutMod4Qreg,filterOutMod5Qreg,filterOutMod7Qreg,filterOutMod11Qreg,scalerOutMod3Q,scalerOutMod4Q,scalerOutMod5Q,scalerOutMod7Q,scalerOutMod11Q);

always @(posedge ck) begin
  scalerOutMod3Ireg <= scalerOutMod3I;
  scalerOutMod3Qreg <= scalerOutMod3Q;
  scalerOutMod4Ireg <= scalerOutMod4I;
  scalerOutMod4Qreg <= scalerOutMod4Q;
  scalerOutMod5Ireg <= scalerOutMod5I;
  scalerOutMod5Qreg <= scalerOutMod5Q;
  scalerOutMod7Ireg <= scalerOutMod7I;
  scalerOutMod7Qreg <= scalerOutMod7Q;
  scalerOutMod11Ireg <= scalerOutMod11I;
  scalerOutMod11Qreg <= scalerOutMod11Q;
end

// Output Converter

OutputConverter#(.NUMBER_OF_MODULI(5),.OUTPUT_WORD_LENGTH(12)) u_OutputConverterI(
  .iMod3(scalerOutMod3Ireg),
  .iMod4(scalerOutMod4Ireg),
  .iMod5(scalerOutMod5Ireg),
  .iMod7(scalerOutMod7Ireg),
  .iMod11(scalerOutMod11Ireg),
  .out(converterOutI)
);

OutputConverter#(.NUMBER_OF_MODULI(5),.OUTPUT_WORD_LENGTH(12)) u_OutputConverterQ(
  .iMod3(scalerOutMod3Qreg),
  .iMod4(scalerOutMod4Qreg),
  .iMod5(scalerOutMod5Qreg),
  .iMod7(scalerOutMod7Qreg),
  .iMod11(scalerOutMod11Qreg),
  .out(converterOutQ)
);

endmodule
