//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module Scaler #(
  parameter NUMBER_OF_MODULI				= pa_Receiver12bits::FIR_NUMBER_OF_MODULI,
  parameter integer MODULI[NUMBER_OF_MODULI - 1:0]	= pa_Receiver12bits::FIR_MODULI
  )(
  input logic [1:0] inMod3,
  input logic [1:0] inMod4,
  input logic [2:0] inMod5,
  input logic [2:0] inMod7,
  input logic [3:0] inMod11,
  output logic [1:0] outMod3,
  output logic [1:0] outMod4,
  output logic [2:0] outMod5,
  output logic [2:0] outMod7,
  output logic [3:0] outMod11
  );
  logic [3:0] scaleInMod3;
  logic [1:0] scaleInMod4;
  logic [3:0] scaleInMod5;
  logic [4:0] scaleInMod7;

  
  logic [1:0] scaleOutMod3;
  logic [1:0] scaleOutMod4;
  logic [2:0] scaleOutMod5;
  logic [2:0] scaleOutMod7;
  logic [3:0] scaleOutMod11;

  logic isNeg;
  logic [3:0] negCorrection[10:0];

  

  assign scaleInMod3 = inMod3 - inMod11;
  assign scaleInMod4 = inMod4 - inMod11;
  assign scaleInMod5 = inMod5 - inMod11;
  assign scaleInMod7 = inMod7 - inMod11;

  logic [1:0] scaleROMMod3 [15:0];
  logic [1:0] scaleROMMod4 [3:0];
  logic [2:0] scaleROMMod5 [15:0];
  logic [2:0] scaleROMMod7 [32:0];
  
  genvar i;
  generate
  for (i = 0; i < 16; i++) begin : la_scaleMod3
    if( i < MODULI[4]) begin
      assign scaleROMMod3[i] = (i * 2)%3;
    end
    else begin
      assign scaleROMMod3[i] = ((i - 1)%3*2)%3;
    end
  end: la_scaleMod3
  endgenerate

  generate
  for (i = 0; i < 4; i++) begin : la_scaleMod4
      assign scaleROMMod4[i] = (i * 3)%4;
  end: la_scaleMod4
  endgenerate

  
  generate
  for (i = 0; i < 16; i++) begin : la_scaleMod5
    if( i < MODULI[2])
      assign scaleROMMod5[i] = (i )%5;
    else 
      assign scaleROMMod5[i] = ((i - 1)%5)%5;
  end: la_scaleMod5
  endgenerate
  
  generate
  for (i = 0; i < 32; i++) begin : la_scaleMod7
    if( i < MODULI[1])
      assign scaleROMMod7[i] = (i * 2)%7;
    else 
      assign scaleROMMod7[i] = ((i - 4)%7*2)%7;
  end: la_scaleMod7
  endgenerate

  generate
  for (i = 0; i < 11; i++) begin : la_negCorrection
    assign negCorrection[i] = (i + 3*4*5*7*11 - 3*4*5*7)%11;
  end : la_negCorrection
  endgenerate

  assign scaleOutMod3 = scaleROMMod3[scaleInMod3];
  assign scaleOutMod4 = scaleROMMod4[scaleInMod4];
  assign scaleOutMod5 = scaleROMMod5[scaleInMod5];
  assign scaleOutMod7 = scaleROMMod7[scaleInMod7];

  BaseExtender u_BaseExtender(scaleOutMod3, scaleOutMod4, scaleOutMod5, scaleOutMod7, isNeg, scaleOutMod11);

  assign outMod3 = scaleOutMod3;
  assign outMod4 = scaleOutMod4;
  assign outMod5 = scaleOutMod5;
  assign outMod7 = scaleOutMod7;
  assign outMod11 = isNeg? negCorrection[scaleOutMod11] :  scaleOutMod11;




endmodule
