//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

module test_Receiver12bits (
  
  );

  localparam integer MODULI [4:0] = {3,4,5,7,11};

  int   errorCnt; //total error count
  int   errorCntTb; //Error count of tb
  int   errorCntAssertion; //Error count assertion
  logic [11:0] signalInI,signalOutI, signalInQ, signalOutQ;
  logic ck;
  // -- Interfaces and signals
  inTest_Receiver12bits uin_Receiver12bits();

  `include "../../rtl/Coefficients.sv"
  `include "../../rtl/NumberOfTaps.sv"

  
  logic [1:0] coeffMod3I [NUMBER_OF_TAPS -1:0];
  logic [1:0] coeffMod4I [NUMBER_OF_TAPS -1:0];
  logic [2:0] coeffMod5I[NUMBER_OF_TAPS -1:0];
  logic [2:0] coeffMod7I[NUMBER_OF_TAPS -1:0];
  logic [3:0] coeffMod11I[NUMBER_OF_TAPS -1:0];

  assign coeffMod3I = $unsigned(COEFFICIENTS3[NUMBER_OF_TAPS -1:0]);
  assign coeffMod4I = $unsigned(COEFFICIENTS4[NUMBER_OF_TAPS -1:0]);
  assign coeffMod5I = $unsigned(COEFFICIENTS5[NUMBER_OF_TAPS -1:0]);
  assign coeffMod7I = $unsigned(COEFFICIENTS7[NUMBER_OF_TAPS -1:0]);
  assign coeffMod11I = $unsigned(COEFFICIENTS11[NUMBER_OF_TAPS -1:0]);

//  Receiver12bits #(.NUMBER_OF_MODULI(5),.MODULI(MODULI)) u_Receiver12bits(signalInI, signalInQ, coeffMod3I, coeffMod4I, coeffMod5I, coeffMod7I, coeffMod11I, ck, signalOutI, signalOutQ);
  Receiver12bits u_Receiver12bits(signalInI, signalInQ, (2*NUMBER_OF_TAPS)'(coeffMod3I), (2*NUMBER_OF_TAPS)'(coeffMod4I), (3*NUMBER_OF_TAPS)'(coeffMod5I), (3*NUMBER_OF_TAPS)'(coeffMod7I), (4*NUMBER_OF_TAPS)'(coeffMod11I), ck, signalOutI, signalOutQ);

  // ------------------------------
  // -- Clock and Reset  
  // ------------------------------
  
  initial begin
    uin_Receiver12bits.arst = 0;
  end
  
  initial begin
    uin_Receiver12bits.clockEnable = 0;
  end
  
  //int cnt;
//-------------------------------------------------------------------------------------------------
//                Output simulation format/results
//-------------------------------------------------------------------------------------------------
  initial
  begin
    $timeformat(-6, 3, "us", 0);
  end   


  // ------------------------------
  // -- DUT and assignments  
  // ------------------------------
  

  logic [11:0] reffQ, reffI;
  assertionsReceiver12bits u_assertionsReceiver12bits(signalOutI, signalOutQ, reffI, reffQ, ck, errorCnt);
  // ------------------------------
  // -- Concurrent assertions
  // ------------------------------
  
  //-- Clocks and reset
  
  // ------------------------------
  // -- Test program  
  // ------------------------------
  //-------------------------------------------------------------------------------------------------
//                Main Stimulus
//-------------------------------------------------------------------------------------------------

logic [11:0] stimuliI [999:0];
logic [11:0] stimuliQ [999:0];
logic [11:0] corrI [1000:0];
logic [11:0] corrQ [1000:0];
`include "signal.v"
`include "results.v"

int i = 0;
initial begin
  @(posedge ck)
  for(i = 1;i <4096; i++) begin
    @(posedge ck);
  end

  fu_printEndStatus();
  $stop;
end


initial begin 
  ck <= 0;
  while(1) begin
    #50;
    ck = ~ck;
  end
end


assign signalInI = stimuliI[i];
assign signalInQ = stimuliQ[i];
assign reffQ = corrQ[i+2];
assign reffI = corrI[i+2];

function void fu_printEndStatus;
    $display("");
    $display("");
    $display($time, " ns:");
    $display("--------------------------------------------------------------------------------");
    $display("--------------------------------------------------------------------------------");
    $display("");
    $display("  SIMULATIONS FINISHED WITH %0d ERROR(S).", errorCnt);
    $display("");
    $display("");
    $display("      -> Check functional coverage.");
    $display("");
    $display("--------------------------------------------------------------------------------");
    $display("--------------------------------------------------------------------------------");
    $display("");
    $display("");
endfunction    

endmodule
