//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 09-28-15                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module BypassMultiplier #( 
//  parameter INCLUDE_CORRELATOR    = pa_Correlator::INCLUDE_CORRELATOR,
  parameter ADDER_WORD_WIDTH	  = 4
 )(
  // The two operands a and b as input signals
  input logic signed   [ADDER_WORD_WIDTH - 1:0] a,
  input logic signed   [ADDER_WORD_WIDTH - 1:0] b,
  // Sum
  output logic         [ADDER_WORD_WIDTH*2 - 1:0] p
  );
  
logic [3:0][3:0] c;
logic [3:0][3:0] c_muxd;
logic [3:0][3:0] s;
logic [3:0][3:0] s_muxd;
logic [3:0][3:0] buffered;
logic signed [3:0] a_neg;

assign a_neg = -a;


generate
for (genvar i = 0; i < 3; i++)begin: la_buffer_first_row
  assign buffered[0][i] = a[i]&&b[1] ? a[i+1] && b[0] : 1'bz;
  assign c_muxd[0][i] = a[i]&&b[1] ? c[0][i] : 0;
  assign s_muxd[0][i] = a[i]&&b[1] ? s[0][i] :a[i+1] && b[0];
end: la_buffer_first_row
endgenerate

generate
for (genvar i = 0; i < 1; i++)begin: la_buffer_first_row_rest
  assign buffered[0][3+i] = a[3] &&b[1] ? a[3]&&b[0] : 1'bz;
  assign c_muxd[0][3+i] = a[3]&& b[1] ? c[0][3+i] : 0;
  assign s_muxd[0][3+i] = a[3] && b[1] ? s[0][3+i] : a[3] &&b[0];
end: la_buffer_first_row_rest
endgenerate

generate
    for(genvar i = 0; i < 3 ; i++)begin: la_buffer_second_row
      assign buffered[1][i] = (a[i]&&b[1+1]) ^ c_muxd[1-1][i] ? s_muxd[1-1][i+1] : 1'bz;
      assign c_muxd[1][i] = (a[i]&&b[1+1]) ^ c_muxd[1-1][i] ? c[1][i] : a[i]&&b[1+1];
      assign s_muxd[1][i] = (a[i]&&b[1+1]) ^ c_muxd[1-1][i] ? s[1][i] : s_muxd[1-1][i+1];
    end: la_buffer_second_row
endgenerate

generate
for (genvar i = 0; i < 1; i++)begin: la_buffer_second_row_rest
  assign buffered[1][3+i] = (a[3] &&b[1+1]) ^ c_muxd[1-1][3+i] ? s_muxd[1-1][3+i] : 1'bz;
  assign c_muxd[1][3+i] = (a[3]&&b[1+1]) ^ c_muxd[1-1][3+i] ? c[1][3+i] : a[3]&&b[1+1];
  assign s_muxd[1][3+i] = (a[3]&&b[1+1]) ^ c_muxd[1-1][3+i] ? s[1][3+i] : s_muxd[1-1][3+i];
end: la_buffer_second_row_rest
endgenerate

//assign buffered[1][5] = (a[3] &&b[1+1]) ^ c_muxd[1-1][5] ? a[3]&&b[1] : 1'bz;
//assign c_muxd[1][5] = (a[3]&&b[1+1]) ^ c_muxd[1-1][5] ? c[1][5] : a[3]&&b[1+1];
//assign s_muxd[1][5] = (a[3]&&b[1+1]) ^ c_muxd[1-1][5] ? s[1][5] : a[3]&&b[1];

generate
    for(genvar i = 0; i < 3 ; i++)begin: la_buffer_third_row
      assign buffered[2][i] = (a_neg[i]&&b[2+1]) ^ c_muxd[2-1][i] ? s_muxd[2-1][i+1] : 1'bz;
      assign c_muxd[2][i] = (a_neg[i]&&b[2+1]) ^ c_muxd[2-1][i] ? c[2][i] : a_neg[i]&&b[2+1];
      assign s_muxd[2][i] = (a_neg[i]&&b[2+1]) ^ c_muxd[2-1][i] ? s[2][i] : s_muxd[2-1][i+1];
    end: la_buffer_third_row
endgenerate

generate
for (genvar i = 0; i < 1; i++)begin: la_buffer_third_row_rest
  assign buffered[2][3+i] = (a_neg[3] &&b[2+1]) ^ c_muxd[2-1][3+i] ? s_muxd[2-1][3+i] : 1'bz;
  assign c_muxd[2][3+i] = (a_neg[3]&&b[2+1]) ^ c_muxd[2-1][3+i] ? c[2][3+i] : a_neg[3]&&b[2+1];
  assign s_muxd[2][3+i] = (a_neg[3]&&b[2+1]) ^ c_muxd[2-1][3+i] ? s[2][3+i] : s_muxd[2-1][3+i];
end: la_buffer_third_row_rest
endgenerate

//assign buffered[2][4] = (a_neg[3] &&b[2+1]) ^ c_muxd[2-1][4] ? a[3]&&b[2] : 1'bz;
//assign c_muxd[2][4] = (a_neg[3]&&b[2+1]) ^ c_muxd[2-1][4] ? c[2][4] : a_neg[3]&&b[2+1];
//assign s_muxd[2][4] = (a_neg[3]&&b[2+1]) ^ c_muxd[2-1][4] ? s[2][4] : a[3]&&b[2];



assign buffered[3][0] =  c_muxd[3-1][0] ? s_muxd[3-1][1] : 1'bz;
assign c_muxd[3][0] =  c_muxd[3-1][0] ? c[3][0] : 1'b0;
assign s_muxd[3][0] =  c_muxd[3-1][0] ? s[3][0] : s_muxd[3-1][1];

generate
for (genvar i = 1; i < 3; i++)begin: la_buffer_last_row
  assign buffered[3][i] =  c_muxd[3][i-1] ^ c_muxd[3-1][i] ? s_muxd[3-1][i+1] : 1'bz;
  assign c_muxd[3][i] =  c_muxd[3][i-1] ^ c_muxd[3-1][i] ? c[3][i] : 0;
  assign s_muxd[3][i] =  c_muxd[3][i-1] ^ c_muxd[3-1][i] ? s[3][i] : s_muxd[3-1][i+1];
end: la_buffer_last_row
endgenerate

assign buffered[3][3] =  c_muxd[3][3-1] ^ c_muxd[3-1][3] ? s_muxd[3-1][3] : 1'bz;
assign c_muxd[3][3] =  c_muxd[3][3-1] ^ c_muxd[3-1][3] ? c[3][3] : c_muxd[3-1][3];
assign s_muxd[3][3] =  c_muxd[3][3-1] ^ c_muxd[3-1][3] ? s[3][3] : s_muxd[3-1][3];


genvar j;
generate
for (genvar i = 0; i < 4; i++)begin: la_Apluss_matrix
  for ( j = 0; j < 4; j++)begin
    APluss1 u_APluss1(buffered[i][j],s[i][j],c[i][j]);
  end
end: la_Apluss_matrix
endgenerate

assign p = {s_muxd[3][3],s_muxd[3][2],s_muxd[3][1],s_muxd[3][0],s_muxd[2][0],s_muxd[1][0],s_muxd[0][0],a[0] && b[0]};

endmodule
