//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module HybridFirCellLinear #(
  parameter WORD_LENGTH		= pa_Receiver12bits::FIR_CELL_WORD_LENGTH,
  parameter MODULO		= pa_Receiver12bits::FIR_CELL_MODULO,
  parameter COEFFICIENT_I	= pa_Receiver12bits::FIR_COEFFICIENT_I
  )(
  input logic [WORD_LENGTH - 1:0] xI,
  input logic [WORD_LENGTH * 2 :0] yI,
  input logic [WORD_LENGTH - 1:0] yCoeff,
  output logic [WORD_LENGTH - 1:0]xOutI,
  output logic [WORD_LENGTH * 2 :0]yOutI
  );

  parameter ADDER_WORD_WIDTH =  WORD_LENGTH*2 +1;
  
  logic [WORD_LENGTH * 2 :0] productI;
  logic [WORD_LENGTH * 2 :0] correctionI;
  logic [WORD_LENGTH * 2 :0] ySumI;
  logic [WORD_LENGTH * 2 :0] yCarryI;



  assign productI = xI * yCoeff;
  CsAdder#(.ADDER_WORD_WIDTH(ADDER_WORD_WIDTH)) u_CsAdderI(productI, yI, correctionI, ySumI, yCarryI);

  always_comb begin
    if(yI[WORD_LENGTH*2] == 1'b1) 
      correctionI = -MODULO*MODULO;
    else
      correctionI = 0;
  end

  assign yOutI = ySumI + (yCarryI << 1);

  assign xOutI = xI;

endmodule
