//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module HybridFirLinear #(
  parameter NUMBER_OF_MODULI								= pa_Receiver16bits::FIR_NUMBER_OF_MODULI,
  //parameter integer MODULI[NUMBER_OF_MODULI -1 :0]					= pa_Receiver16bits::FIR_MODULI,
  parameter NUMBER_OF_TAPS								= pa_Receiver16bits::FIR_NUMBER_OF_TAPS
//  parameter integer COEFFICIENTS[NUMBER_OF_MODULI - 1:0][NUMBER_OF_TAPS - 1:0]		= pa_Receiver16bits::FIR_COEFFICIENTS
  )(
  input logic ck,
  input logic [2:0] iMod5I,
  input logic [2:0] iMod7I,
  input logic [3:0] iMod11I,
  input logic [3:0] iMod13I,
  input logic [3:0] iMod16I,
  input logic [2:0] coeffMod5I [NUMBER_OF_MODULI -1:0],
  input logic [2:0] coeffMod7I [NUMBER_OF_MODULI -1:0],
  input logic [3:0] coeffMod11I [NUMBER_OF_MODULI -1:0],
  input logic [3:0] coeffMod13I [NUMBER_OF_MODULI -1:0],
  input logic [3:0] coeffMod16I [NUMBER_OF_MODULI -1:0],
  output logic [2:0] oMod5I,
  output logic [2:0] oMod7I,
  output logic [3:0] oMod11I,
  output logic [3:0] oMod13I,
  output logic [3:0] oMod16I
  );

// MODULO 5 FIR filter
 
logic [2:0] xMod5I[NUMBER_OF_TAPS:0];
logic [2:0] xMod5Inext[NUMBER_OF_TAPS:0];
logic [2:0] xMod5Ireg[NUMBER_OF_TAPS:0];
logic [6:0] yMod5I[NUMBER_OF_TAPS:0];

genvar i;
generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod5
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(5)) u_HybridCellMod5(xMod5I[i],yMod5I[i],coeffMod5I[i],xMod5Ireg[i+1],yMod5I[i+1]);
    assign xMod5I[i+1] = xMod5Inext[i+1];
end: la_firMod5
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod5
    xMod5Inext[i] <= xMod5Ireg[i];
  end: la_regMod5
end

assign xMod5I[0] = iMod5I;
assign yMod5I[0] = 7'b0000000;

InputConverter #(.WORD_LENGTH(8),.MODULO(5),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod5I({1'b0,yMod5I[NUMBER_OF_TAPS]},oMod5I);

// MODULO 4 FIR filter

logic [NUMBER_OF_TAPS:0][3:0] xMod16I;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Inext;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Iprod;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Ireg;
logic [NUMBER_OF_TAPS:0][3:0] yMod16I;


generate
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod16
    assign xMod16Iprod[i] = xMod16I[i]*coeffMod16I[i];
    assign xMod16Ireg[i+1] = xMod16I[i];
    assign yMod16I[i+1] = yMod16I[i] + xMod16Iprod[i];
    assign xMod16I[i+1] = xMod16Inext[i+1];
  end: la_firMod16
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod16
    xMod16Inext[i] <= xMod16Ireg[i];
  end: la_regMod16
end


assign xMod16I[0] = iMod16I;
assign yMod16I[0] = 5'b000;


assign oMod16I  = yMod16I[NUMBER_OF_TAPS];

// MODULO 7 FIR filter 

logic [NUMBER_OF_TAPS:0][2:0] xMod7I;
logic [2:0] xMod7Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][2:0] xMod7Ireg;
logic [NUMBER_OF_TAPS:0][6:0] yMod7I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod7
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(7)) u_HybridCellMod7(xMod7I[i],yMod7I[i],coeffMod7I[i],xMod7Ireg[i+1],yMod7I[i+1]);
    assign xMod7I[i+1] = xMod7Inext[i+1];
end: la_firMod7
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod7
    xMod7Inext[i] <= xMod7Ireg[i];
  end: la_regMod7
end

assign xMod7I[0] = iMod7I;
assign yMod7I[0] = 7'b0000000;

InputConverter #(.WORD_LENGTH(8),.MODULO(7),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod7I({1'b0,yMod7I[NUMBER_OF_TAPS]},oMod7I);

// MODULO 11 FIR filter 

logic [NUMBER_OF_TAPS:0][3:0] xMod11I;
logic [NUMBER_OF_TAPS:0][3:0] xMod11Ireg;
logic [3:0] xMod11Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][8:0] yMod11I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod11
    HybridFirCellLinear#(.WORD_LENGTH(4),.MODULO(11)) u_HybridCellMod11(xMod11I[i],yMod11I[i],coeffMod11I[i],xMod11Ireg[i+1],yMod11I[i+1]);
    assign xMod11I[i+1] = xMod11Inext[i+1];
end: la_firMod11
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod11
    xMod11Inext[i] <= xMod11Ireg[i];
  end: la_regMod11
end

assign xMod11I[0] = iMod11I;
assign yMod11I[0] = 9'b000000000;
  
InputConverter #(.WORD_LENGTH(10),.MODULO(11),.MODULO_WORD_LENGTH(4)) u_ResidueConverterMod11I({1'b0,yMod11I[NUMBER_OF_TAPS]},oMod11I);

// MODULO 13 FIR filter 

logic [NUMBER_OF_TAPS:0][3:0] xMod13I;
logic [3:0] xMod13Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][3:0] xMod13Ireg;
logic [NUMBER_OF_TAPS:0][8:0] yMod13I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod13
    HybridFirCellLinear#(.WORD_LENGTH(4),.MODULO(13)) u_HybridCellMod13(xMod13I[i],yMod13I[i],coeffMod13I[i],xMod13Ireg[i+1],yMod13I[i+1]);
    assign xMod13I[i+1] = xMod13Inext[i+1];
end: la_firMod13
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod13
    xMod13Inext[i] <= xMod13Ireg[i];
  end: la_regMod13
end

assign xMod13I[0] = iMod13I;
assign yMod13I[0] = 9'b000000000;

InputConverter #(.WORD_LENGTH(10),.MODULO(13),.MODULO_WORD_LENGTH(4)) u_ResidueConverterMod13I({1'b0,yMod13I[NUMBER_OF_TAPS]},oMod13I);
endmodule
