//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module OutputConverter #(
  parameter NUMBER_OF_MODULI			    = pa_Receiver24bits::OUTPUT_CONVERTER_NUMBER_OF_MODULI,
  parameter OUTPUT_WORD_LENGTH			    = pa_Receiver24bits::OUTPUT_CONVERTER_WORD_LENGTH
  )(
  input logic [3:0] iMod13,
  input logic [3:0] iMod16,
  input logic [2:0] iMod5,
  input logic [2:0] iMod7,
  input logic [3:0] iMod11,
  input logic [4:0] iMod17,
  input logic [3:0] iMod9,
  output logic [23:0] out
  );
  localparam PROD_OF_MODULI = 13*16*5*7*11*17*9;
  logic [24  : 0] weightedProduct [NUMBER_OF_MODULI - 1: 0];
  logic [24  : 0] weightningLUTmod13 [13: 0];
  logic [24  : 0] weightningLUTmod16 [16: 0];
  logic [24  : 0] weightningLUTmod5 [5: 0];
  logic [24  : 0] weightningLUTmod7 [7: 0];
  logic [24  : 0] weightningLUTmod11 [11: 0];
  logic [24  : 0] weightningLUTmod17 [17: 0];
  logic [24  : 0] weightningLUTmod9 [9: 0];
  

  // ROM for Moduli 13

  genvar i;
  generate
  for(i = 0; i < 13; i++)begin: la_WeightROMmod13
      assign weightningLUTmod13[i] = (i*16*5*7*11*9*17*11)%PROD_OF_MODULI;
  end: la_WeightROMmod13	
  endgenerate
  
  assign weightedProduct[0] = weightningLUTmod13[iMod13]; 
  
  // ROM for Moduli 16

  generate
  for(i = 0; i < 16; i++)begin: la_WeightROMmod16
      assign weightningLUTmod16[i] = (i*13*5*7*11*17*9*13)%PROD_OF_MODULI;
  end: la_WeightROMmod16
  endgenerate
  
  assign weightedProduct[1] = weightningLUTmod16[iMod16]; 

  // ROM for Moduli 5

  generate
  for(i = 0; i < 5; i++)begin: la_WeightROMmod5
      assign weightningLUTmod5[i] = (i*13*16*7*11*17*9*2)%PROD_OF_MODULI;
  end: la_WeightROMmod5
  endgenerate

  assign weightedProduct[2] = weightningLUTmod5[iMod5]; 

  // ROM for Moduli 7

  generate
  for(i = 0; i < 7; i++)begin: la_WeightROMmod7
      assign weightningLUTmod7[i] = (i*13*16*5*11*17*9*3)%PROD_OF_MODULI;
  end: la_WeightROMmod7
  endgenerate

  assign weightedProduct[3] = weightningLUTmod7[iMod7]; 

  // ROM for Moduli 11

  generate
  for(i = 0; i < 11; i++)begin: la_WeightROMmod11
      assign weightningLUTmod11[i] = (i*13*16*5*7*17*9*6)%PROD_OF_MODULI;
  end: la_WeightROMmod11
  endgenerate

  assign weightedProduct[4] = weightningLUTmod11[iMod11]; 

  // ROM for Moduli 17

  generate
  for(i = 0; i < 17; i++)begin: la_WeightROMmod17
      assign weightningLUTmod17[i] = (i*13*16*5*7*11*9*7)%PROD_OF_MODULI;
  end: la_WeightROMmod17
  endgenerate

  assign weightedProduct[5] = weightningLUTmod17[iMod17]; 


  // ROM for Moduli 9

  generate
  for(i = 0; i < 9; i++)begin: la_WeightROMmod9
      assign weightningLUTmod9[i] = (i*13*16*5*7*11*17*5)%PROD_OF_MODULI;
  end: la_WeightROMmod9
  endgenerate

  assign weightedProduct[6] = weightningLUTmod9[iMod9]; 



  // Carry Save Adder Tree
  logic [24:0] AdderTreeStage1s, AdderTreeStage1c;
  logic [24:0] AdderTreeStage2s, AdderTreeStage2c;
  logic [25:0] AdderTreeStage3s, AdderTreeStage3c;
  logic [25:0] AdderTreeStage4s, AdderTreeStage4c;
  logic [26:0] AdderTreeStage5s, AdderTreeStage5c;

  CsAdder#(.ADDER_WORD_WIDTH(25)) u_CsAdderTreeStage1(weightedProduct[4], weightedProduct[3], weightedProduct[2], AdderTreeStage1s, AdderTreeStage1c);
  CsAdder#(.ADDER_WORD_WIDTH(25)) u_CsAdderTreeStage2(weightedProduct[5], weightedProduct[1], weightedProduct[0], AdderTreeStage2s, AdderTreeStage2c);
  CsAdder#(.ADDER_WORD_WIDTH(26)) u_CsAdderTreeStage3({1'b0,AdderTreeStage2s}, {AdderTreeStage2c,1'b0}, {AdderTreeStage1c, 1'b0}, AdderTreeStage3s, AdderTreeStage3c);
  CsAdder#(.ADDER_WORD_WIDTH(26)) u_CsAdderTreeStage4(AdderTreeStage3s, {1'b0,AdderTreeStage1s}, {1'b0, weightedProduct[6]}, AdderTreeStage4s, AdderTreeStage4c);
  CsAdder#(.ADDER_WORD_WIDTH(27)) u_CsAdderTreeStage5({1'b0,AdderTreeStage4s}, {AdderTreeStage3c,1'b0}, {AdderTreeStage4c, 1'b0}, AdderTreeStage5s, AdderTreeStage5c);


  //Correction Rom 

  logic[27:0] correction;

  assign correction = - 3 * 13*16*5*7*11*17*9; //max correction
  //Correction adder
  
  logic[27:0] corrAdders, corrAdderc, binaryNumber1; 
  logic[26:0] binaryNumber2, binaryNumber3;
  
  CsAdder#(.ADDER_WORD_WIDTH(28)) u_CsAdderCorrAdder({1'b0, AdderTreeStage5s},{AdderTreeStage5c, 1'b0} , correction, corrAdders, corrAdderc);
  
  assign binaryNumber1 = corrAdders + (corrAdderc << 1);

  always @(*)
  begin
    if(binaryNumber1[27] == 1'b0)begin
      binaryNumber2 = binaryNumber1 - (2*13*16*5*7*11*17*9);
    end
    else begin
      binaryNumber2 = binaryNumber1 + (2*13*16*5*7*11*17*9);
    end
  end
  
  logic [24:0] outPositive;
  always @(*)
  begin
    if(binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b1)
      binaryNumber3 = binaryNumber2 + (13*16*5*7*11*17*9);
    else if (binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b0)
      binaryNumber3 = binaryNumber2 - (13*16*5*7*11*17*9);
    else if(binaryNumber1[27] == 1'b1 && binaryNumber2[26] == 1'b0)
      binaryNumber3 = binaryNumber2 - (13*16*5*7*11*17*9);
    else
      binaryNumber3 = binaryNumber2 + (13*16*5*7*11*17*9);
  end

  always @(*)
  begin
    if(binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b0 && binaryNumber3[26] == 1'b0)
      outPositive = binaryNumber3;
    else if (binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b0 && binaryNumber3[26] == 1'b1)
      outPositive = binaryNumber2;
    else if (binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b1 && binaryNumber3[26] == 1'b0)
      outPositive = binaryNumber3;
    else if (binaryNumber1[27] == 1'b0 && binaryNumber2[26] == 1'b1 && binaryNumber3[26] == 1'b1)
      outPositive = binaryNumber1;
    else if (binaryNumber1[27] == 1'b1 && binaryNumber2[26] == 1'b0 && binaryNumber3[26] == 1'b0)
      outPositive = binaryNumber3;
    else if (binaryNumber1[27] == 1'b1 && binaryNumber2[26] == 1'b0 && binaryNumber3[26] == 1'b1)
      outPositive = binaryNumber2;
    else if (binaryNumber1[27] == 1'b1 && binaryNumber2[26] == 1'b1 && binaryNumber3[26] == 1'b0)
      outPositive = binaryNumber3;
    else
      outPositive = 0;
  end 

  assign out = outPositive > ((13*16*5*7*11*17*9)/2) ? outPositive - (13*16*5*7*11*17*9) : outPositive;
  
endmodule
