//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

  `include "NumberOfTaps.sv"

module Receiver24bits #( 
  parameter INCLUDE_RECEIVER16BITS    = pa_Receiver24bits::INCLUDE_RECEIVER16BITS,
  parameter NUMBER_OF_MODULI	      = pa_Receiver24bits::FIR_NUMBER_OF_MODULI
  //parameter integer MODULI [NUMBER_OF_MODULI - 1:0]      = pa_Receiver24bits::FIR_MODULI 
  )(
  input logic [23:0] signalInI,
  input logic [23:0] signalInQ,
  input logic [2:0] coeffMod5 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod16 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod7 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod11 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod13 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod17 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod9 [NUMBER_OF_TAPS -1:0],
  input ck, 
  output logic[23:0] converterOutI,
  output logic[23:0] converterOutQ
  );
  localparam integer MODULI [6:0] = {5, 16, 7, 11, 13, 17, 9};


  logic [2:0] convOutMod5I, convOutMod5Q; 
  logic [2:0] convOutMod7I, convOutMod7Q;
  logic [3:0] convOutMod11I, convOutMod11Q;
  logic [3:0] convOutMod13I, convOutMod13Q;
  logic [3:0] convOutMod16I, convOutMod16Q;
  logic [4:0] convOutMod17I, convOutMod17Q;
  logic [3:0] convOutMod9I, convOutMod9Q;

  logic [2:0] convOutMod5Ireg, convOutMod5Qreg; 
  logic [2:0] convOutMod7Ireg, convOutMod7Qreg;
  logic [3:0] convOutMod11Ireg, convOutMod11Qreg;
  logic [3:0] convOutMod13Ireg, convOutMod13Qreg;
  logic [3:0] convOutMod16Ireg, convOutMod16Qreg;
  logic [4:0] convOutMod17Ireg, convOutMod17Qreg;
  logic [3:0] convOutMod9Ireg, convOutMod9Qreg;


  logic [2:0] filterOutMod5I, filterOutMod5Q; 
  logic [2:0] filterOutMod7I, filterOutMod7Q;
  logic [3:0] filterOutMod11I, filterOutMod11Q;
  logic [3:0] filterOutMod13I, filterOutMod13Q;
  logic [3:0] filterOutMod16I, filterOutMod16Q;
  logic [4:0] filterOutMod17I, filterOutMod17Q;
  logic [3:0] filterOutMod9I, filterOutMod9Q;

  logic [2:0] filterOutMod5Ireg, filterOutMod5Qreg; 
  logic [2:0] filterOutMod7Ireg, filterOutMod7Qreg;
  logic [3:0] filterOutMod11Ireg, filterOutMod11Qreg;
  logic [3:0] filterOutMod13Ireg, filterOutMod13Qreg;
  logic [3:0] filterOutMod16Ireg, filterOutMod16Qreg;
  logic [4:0] filterOutMod17Ireg, filterOutMod17Qreg;
  logic [3:0] filterOutMod9Ireg, filterOutMod9Qreg;
// Input Converters
 
InputConverter #(.WORD_LENGTH(24),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5I (
    .in (signalInI),
    .out (convOutMod5I)
    );
assign convOutMod16I = signalInI[3:0];
InputConverter #(.WORD_LENGTH(24),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7I (
    .in (signalInI),
    .out (convOutMod7I)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(11),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod11I (
    .in (signalInI),
    .out (convOutMod11I)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(13),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod13I (
    .in (signalInI),
    .out (convOutMod13I)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(17),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod17I (
    .in (signalInI),
    .out (convOutMod17I)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(9),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod9I (
    .in (signalInI),
    .out (convOutMod9I)
    );

InputConverter #(.WORD_LENGTH(24),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5Q (
    .in (signalInQ),
    .out (convOutMod5Q)
    );
assign convOutMod16Q = signalInQ[3:0];
InputConverter #(.WORD_LENGTH(24),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7Q (
    .in (signalInQ),
    .out (convOutMod7Q)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(11),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod11Q (
    .in (signalInQ),
    .out (convOutMod11Q)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(13),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod13Q (
    .in (signalInQ),
    .out (convOutMod13Q)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(17),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod17Q (
    .in (signalInQ),
    .out (convOutMod17Q)
    );
InputConverter #(.WORD_LENGTH(24),
  .MODULO(9),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod9Q (
    .in (signalInQ),
    .out (convOutMod9Q)
    );

always @(posedge ck) begin
  convOutMod5Ireg <= convOutMod5I;
  convOutMod5Qreg <= convOutMod5Q;
  convOutMod7Ireg <= convOutMod7I;
  convOutMod7Qreg <= convOutMod7Q;
  convOutMod11Ireg <= convOutMod11I;
  convOutMod11Qreg <= convOutMod11Q;
  convOutMod13Ireg <= convOutMod13I;
  convOutMod13Qreg <= convOutMod13Q;
  convOutMod16Ireg <= convOutMod16I;
  convOutMod16Qreg <= convOutMod16Q;
  convOutMod17Ireg <= convOutMod17I;
  convOutMod17Qreg <= convOutMod17Q;
  convOutMod9Ireg <= convOutMod9I;
  convOutMod9Qreg <= convOutMod9Q;
end

// FIR Filter  
  
HybridFirLinear #(.NUMBER_OF_MODULI(7), .MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearI(.ck(ck), .iMod5I(convOutMod5Ireg), .iMod7I(convOutMod7Ireg), .iMod11I(convOutMod11Ireg), .iMod13I(convOutMod13Ireg), .iMod16I(convOutMod16Ireg), .iMod17I(convOutMod17Ireg), .iMod9I(convOutMod9Ireg), .coeffMod5(coeffMod5), .coeffMod16(coeffMod16), .coeffMod7(coeffMod7), .coeffMod11(coeffMod11), .coeffMod13(coeffMod13), .coeffMod17(coeffMod17), .coeffMod9(coeffMod9), .oMod5I(filterOutMod5I), .oMod7I(filterOutMod7I), .oMod11I(filterOutMod11I), .oMod13I(filterOutMod13I), .oMod16I(filterOutMod16I), .oMod17I(filterOutMod17I), .oMod9I(filterOutMod9I));


HybridFirLinear #(.NUMBER_OF_MODULI(7), .MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearQ(.ck(ck), .iMod5I(convOutMod5Qreg), .iMod7I(convOutMod7Qreg), .iMod11I(convOutMod11Qreg), .iMod13I(convOutMod13Qreg), .iMod16I(convOutMod16Qreg), .iMod17I(convOutMod17Qreg), .iMod9I(convOutMod9Qreg),  .coeffMod5(coeffMod5), .coeffMod16(coeffMod16), .coeffMod7(coeffMod7), .coeffMod11(coeffMod11), .coeffMod13(coeffMod13), .coeffMod17(coeffMod17), .coeffMod9(coeffMod9), .oMod5I(filterOutMod5Q), .oMod7I(filterOutMod7Q), .oMod11I(filterOutMod11Q), .oMod13I(filterOutMod13Q), .oMod16I(filterOutMod16Q), .oMod17I(filterOutMod17Q), .oMod9I(filterOutMod9Q));


// Output Converter

always @(posedge ck) begin
  filterOutMod5Ireg <= filterOutMod5I;
  filterOutMod5Qreg <= filterOutMod5Q;
  filterOutMod7Ireg <= filterOutMod7I;
  filterOutMod7Qreg <= filterOutMod7Q;
  filterOutMod11Ireg <= filterOutMod11I;
  filterOutMod11Qreg <= filterOutMod11Q;
  filterOutMod13Ireg <= filterOutMod13I;
  filterOutMod13Qreg <= filterOutMod13Q;
  filterOutMod16Ireg <= filterOutMod16I;
  filterOutMod16Qreg <= filterOutMod16Q;
  filterOutMod17Ireg <= filterOutMod17I;
  filterOutMod17Qreg <= filterOutMod17Q;
  filterOutMod9Ireg <= filterOutMod9I;
  filterOutMod9Qreg <= filterOutMod9Q;
end

OutputConverter#(.NUMBER_OF_MODULI(7),.OUTPUT_WORD_LENGTH(16)) u_OutputConverterI(
  .iMod5(filterOutMod5Ireg),
  .iMod7(filterOutMod7Ireg),
  .iMod11(filterOutMod11Ireg),
  .iMod13(filterOutMod13Ireg),
  .iMod16(filterOutMod16Ireg),
  .iMod17(filterOutMod17Ireg),
  .iMod9(filterOutMod9Ireg),
  .out(converterOutI)
);

OutputConverter#(.NUMBER_OF_MODULI(7),.OUTPUT_WORD_LENGTH(16)) u_OutputConverterQ(
  .iMod5(filterOutMod5Qreg),
  .iMod7(filterOutMod7Qreg),
  .iMod11(filterOutMod11Qreg),
  .iMod13(filterOutMod13Qreg),
  .iMod16(filterOutMod16Qreg),
  .iMod17(filterOutMod17Qreg),
  .iMod9(filterOutMod9Qreg),
  .out(converterOutQ)
);

endmodule
