//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module HybridFirLinear #(
  parameter NUMBER_OF_MODULI								= pa_Receiver32bits::FIR_NUMBER_OF_MODULI,
  parameter integer MODULI[NUMBER_OF_MODULI -1 :0]					= pa_Receiver32bits::FIR_MODULI,
  parameter NUMBER_OF_TAPS								= pa_Receiver32bits::FIR_NUMBER_OF_TAPS
//  parameter integer COEFFICIENTS[NUMBER_OF_MODULI - 1:0][NUMBER_OF_TAPS - 1:0]		= pa_Receiver24bits::FIR_COEFFICIENTS
  )(
  input logic ck,
  input logic [1:0] iMod3I,
  input logic [2:0] iMod5I,
  input logic [2:0] iMod7I,
  input logic [3:0] iMod13I,
  input logic [3:0] iMod16I,
  input logic [4:0] iMod17I,
  input logic [4:0] iMod19I,
  input logic [4:0] iMod23I,
  input logic [4:0] iMod29I,
  input logic [1:0] coeffMod3 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod5 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod16 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod7 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod13 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod17 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod19 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod23 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod29 [NUMBER_OF_TAPS -1:0],
  output logic [1:0] oMod3I,
  output logic [2:0] oMod5I,
  output logic [2:0] oMod7I,
  output logic [3:0] oMod13I,
  output logic [3:0] oMod16I,
  output logic [4:0] oMod17I,
  output logic [4:0] oMod19I,
  output logic [4:0] oMod23I,
  output logic [4:0] oMod29I
  );

`include "Coefficients.sv"

// MODULO 3 FIR filter
 
logic [1:0] xMod3I[NUMBER_OF_TAPS:0];
logic [1:0] xMod3Inext[NUMBER_OF_TAPS:0];
logic [1:0] xMod3Ireg[NUMBER_OF_TAPS:0];
logic [4:0] yMod3I[NUMBER_OF_TAPS:0];

genvar i;
generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod3
    HybridFirCellLinear#(.WORD_LENGTH(2),.MODULO(3)) u_HybridCellMod3(xMod3I[i],yMod3I[i],coeffMod3[i],xMod3Ireg[i+1],yMod3I[i+1]);
    assign xMod3I[i+1] = xMod3Inext[i+1];
end: la_firMod3
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod3
    xMod3Inext[i] <= xMod3Ireg[i];
  end: la_regMod3
end

assign xMod3I[0] = iMod3I;
assign yMod3I[0] = 5'b00000;

InputConverter #(.WORD_LENGTH(6),.MODULO(3),.MODULO_WORD_LENGTH(2)) u_ResidueConverterMod3I({1'b0,yMod3I[NUMBER_OF_TAPS]},oMod3I);

// MODULO 5 FIR filter
 
logic [2:0] xMod5I[NUMBER_OF_TAPS:0];
logic [2:0] xMod5Inext[NUMBER_OF_TAPS:0];
logic [2:0] xMod5Ireg[NUMBER_OF_TAPS:0];
logic [6:0] yMod5I[NUMBER_OF_TAPS:0];

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod5
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(5)) u_HybridCellMod5(xMod5I[i],yMod5I[i],coeffMod5[i],xMod5Ireg[i+1],yMod5I[i+1]);
    assign xMod5I[i+1] = xMod5Inext[i+1];
end: la_firMod5
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod5
    xMod5Inext[i] <= xMod5Ireg[i];
  end: la_regMod5
end

assign xMod5I[0] = iMod5I;
assign yMod5I[0] = 7'b0000000;

InputConverter #(.WORD_LENGTH(8),.MODULO(5),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod5I({1'b0,yMod5I[NUMBER_OF_TAPS]},oMod5I);

// MODULO 16 FIR filter

logic [NUMBER_OF_TAPS:0][3:0] xMod16I;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Inext;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Iprod;
logic [NUMBER_OF_TAPS:0][3:0] xMod16Ireg;
logic [NUMBER_OF_TAPS:0][3:0] yMod16I;


generate
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod16
    assign xMod16Iprod[i] = xMod16I[i]*coeffMod16[i];
    assign xMod16Ireg[i+1] = xMod16I[i];
    assign yMod16I[i+1] = yMod16I[i] + xMod16Iprod[i];
    assign xMod16I[i+1] = xMod16Inext[i+1];
  end: la_firMod16
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod16
    xMod16Inext[i] <= xMod16Ireg[i];
  end: la_regMod16
end


assign xMod16I[0] = iMod16I;
assign yMod16I[0] = 5'b000;


assign oMod16I  = yMod16I[NUMBER_OF_TAPS];

// MODULO 7 FIR filter 

logic [NUMBER_OF_TAPS:0][2:0] xMod7I;
logic [2:0] xMod7Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][2:0] xMod7Ireg;
logic [NUMBER_OF_TAPS:0][6:0] yMod7I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod7
    HybridFirCellLinear#(.WORD_LENGTH(3),.MODULO(7)) u_HybridCellMod7(xMod7I[i],yMod7I[i],coeffMod7[i],xMod7Ireg[i+1],yMod7I[i+1]);
    assign xMod7I[i+1] = xMod7Inext[i+1];
end: la_firMod7
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod7
    xMod7Inext[i] <= xMod7Ireg[i];
  end: la_regMod7
end

assign xMod7I[0] = iMod7I;
assign yMod7I[0] = 7'b0000000;

InputConverter #(.WORD_LENGTH(8),.MODULO(7),.MODULO_WORD_LENGTH(3)) u_ResidueConverterMod7I({1'b0,yMod7I[NUMBER_OF_TAPS]},oMod7I);

// MODULO 13 FIR filter 

logic [NUMBER_OF_TAPS:0][3:0] xMod13I;
logic [3:0] xMod13Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][3:0] xMod13Ireg;
logic [NUMBER_OF_TAPS:0][8:0] yMod13I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod13
    HybridFirCellLinear#(.WORD_LENGTH(4),.MODULO(13)) u_HybridCellMod13(xMod13I[i],yMod13I[i],coeffMod13[i],xMod13Ireg[i+1],yMod13I[i+1]);
    assign xMod13I[i+1] = xMod13Inext[i+1];
end: la_firMod13
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod13
    xMod13Inext[i] <= xMod13Ireg[i];
  end: la_regMod13
end

assign xMod13I[0] = iMod13I;
assign yMod13I[0] = 9'b000000000;

InputConverter #(.WORD_LENGTH(10),.MODULO(13),.MODULO_WORD_LENGTH(4)) u_ResidueConverterMod13I({1'b0,yMod13I[NUMBER_OF_TAPS]},oMod13I);

// MODULO 17 FIR filter 

logic [NUMBER_OF_TAPS:0][4:0] xMod17I;
logic [4:0] xMod17Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][4:0] xMod17Ireg;
logic [NUMBER_OF_TAPS:0][10:0] yMod17I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod17
    HybridFirCellLinear#(.WORD_LENGTH(5),.MODULO(17)) u_HybridCellMod17(xMod17I[i],yMod17I[i],coeffMod17[i],xMod17Ireg[i+1],yMod17I[i+1]);
    assign xMod17I[i+1] = xMod17Inext[i+1];
end: la_firMod17
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod17
    xMod17Inext[i] <= xMod17Ireg[i];
  end: la_regMod17
end

assign xMod17I[0] = iMod17I;
assign yMod17I[0] = 11'b00000000000;

InputConverter #(.WORD_LENGTH(12),.MODULO(17),.MODULO_WORD_LENGTH(5)) u_ResidueConverterMod17I({1'b0,yMod17I[NUMBER_OF_TAPS]},oMod17I);

// MODULO 19 FIR filter 

logic [NUMBER_OF_TAPS:0][4:0] xMod19I;
logic [4:0] xMod19Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][4:0] xMod19Ireg;
logic [NUMBER_OF_TAPS:0][10:0] yMod19I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod19
    HybridFirCellLinear#(.WORD_LENGTH(5),.MODULO(19)) u_HybridCellMod19(xMod19I[i],yMod19I[i],coeffMod19[i],xMod19Ireg[i+1],yMod19I[i+1]);
    assign xMod19I[i+1] = xMod19Inext[i+1];
end: la_firMod19
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod19
    xMod19Inext[i] <= xMod19Ireg[i];
  end: la_regMod19
end

assign xMod19I[0] = iMod19I;
assign yMod19I[0] = 11'b00000000000;

InputConverter #(.WORD_LENGTH(12),.MODULO(19),.MODULO_WORD_LENGTH(5)) u_ResidueConverterMod19I({1'b0,yMod19I[NUMBER_OF_TAPS]},oMod19I);

// MODULO 23 FIR filter 

logic [NUMBER_OF_TAPS:0][4:0] xMod23I;
logic [4:0] xMod23Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][4:0] xMod23Ireg;
logic [NUMBER_OF_TAPS:0][10:0] yMod23I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod23
    HybridFirCellLinear#(.WORD_LENGTH(5),.MODULO(23)) u_HybridCellMod23(xMod23I[i],yMod23I[i],coeffMod23[i],xMod23Ireg[i+1],yMod23I[i+1]);
    assign xMod23I[i+1] = xMod23Inext[i+1];
end: la_firMod23
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod23
    xMod23Inext[i] <= xMod23Ireg[i];
  end: la_regMod23
end

assign xMod23I[0] = iMod23I;
assign yMod23I[0] = 11'b00000000000;

InputConverter #(.WORD_LENGTH(12),.MODULO(23),.MODULO_WORD_LENGTH(5)) u_ResidueConverterMod23I({1'b0,yMod23I[NUMBER_OF_TAPS]},oMod23I);


// MODULO 29 FIR filter 

logic [NUMBER_OF_TAPS:0][4:0] xMod29I;
logic [4:0] xMod29Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][4:0] xMod29Ireg;
logic [NUMBER_OF_TAPS:0][10:0] yMod29I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod29
    HybridFirCellLinear#(.WORD_LENGTH(5),.MODULO(29)) u_HybridCellMod29(xMod29I[i],yMod29I[i],coeffMod29[i],xMod29Ireg[i+1],yMod29I[i+1]);
    assign xMod29I[i+1] = xMod29Inext[i+1];
end: la_firMod29
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod29
    xMod29Inext[i] <= xMod29Ireg[i];
  end: la_regMod29
end

assign xMod29I[0] = iMod29I;
assign yMod29I[0] = 11'b00000000000;

InputConverter #(.WORD_LENGTH(12),.MODULO(29),.MODULO_WORD_LENGTH(5)) u_ResidueConverterMod29I({1'b0,yMod29I[NUMBER_OF_TAPS]},oMod29I);



endmodule
