//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module OutputConverter #(
  parameter NUMBER_OF_MODULI			    = pa_Receiver32bits::OUTPUT_CONVERTER_NUMBER_OF_MODULI,
  parameter OUTPUT_WORD_LENGTH			    = pa_Receiver32bits::OUTPUT_CONVERTER_WORD_LENGTH
  )(
  input logic [3:0] iMod13,
  input logic [3:0] iMod16,
  input logic [1:0] iMod3,
  input logic [2:0] iMod5,
  input logic [2:0] iMod7,
  input logic [4:0] iMod17,
  input logic [4:0] iMod19,
  input logic [4:0] iMod23,
  input logic [4:0] iMod29,
  output logic [31:0] out
  );
  localparam longint PROD_OF_MODULI = 13*16*3*5*7*17*19*23*29;
  logic [32  : 0] weightedProduct [NUMBER_OF_MODULI - 1: 0];
  logic [32  : 0] weightningLUTmod13 [13: 0];
  logic [32  : 0] weightningLUTmod16 [16: 0];
  logic [32  : 0] weightningLUTmod5 [5: 0];
  logic [32  : 0] weightningLUTmod3 [3: 0];
  logic [32  : 0] weightningLUTmod7 [7: 0];
  logic [32  : 0] weightningLUTmod17 [17: 0];
  logic [32  : 0] weightningLUTmod19 [19: 0];
  logic [32  : 0] weightningLUTmod23 [23: 0];
  logic [32  : 0] weightningLUTmod29 [29: 0];
  

  // ROM for Moduli 13

  genvar i;
  generate
  for(i = 0; i < 13; i++)begin: la_WeightROMmod13
      assign weightningLUTmod13[i] = (i*PROD_OF_MODULI/13*7)%PROD_OF_MODULI;
  end: la_WeightROMmod13	
  endgenerate
  
  assign weightedProduct[0] = weightningLUTmod13[iMod13]; 
  
  // ROM for Moduli 16

  generate
  for(i = 0; i < 16; i++)begin: la_WeightROMmod16
      assign weightningLUTmod16[i] = (i*PROD_OF_MODULI/16*13)%PROD_OF_MODULI;
  end: la_WeightROMmod16
  endgenerate
  
  assign weightedProduct[1] = weightningLUTmod16[iMod16]; 

  // ROM for Moduli 3

  generate
  for(i = 0; i < 3; i++)begin: la_WeightROMmod3
      assign weightningLUTmod3[i] = (i*PROD_OF_MODULI/3*1)%PROD_OF_MODULI;
  end: la_WeightROMmod3
  endgenerate

  assign weightedProduct[2] = weightningLUTmod3[iMod3]; 


  // ROM for Moduli 5

  generate
  for(i = 0; i < 5; i++)begin: la_WeightROMmod5
      assign weightningLUTmod5[i] = (i*PROD_OF_MODULI/5*2)%PROD_OF_MODULI;
  end: la_WeightROMmod5
  endgenerate

  assign weightedProduct[3] = weightningLUTmod5[iMod5]; 

  // ROM for Moduli 7

  generate
  for(i = 0; i < 7; i++)begin: la_WeightROMmod7
      assign weightningLUTmod7[i] = (i*PROD_OF_MODULI/7*5)%PROD_OF_MODULI;
  end: la_WeightROMmod7
  endgenerate

  assign weightedProduct[4] = weightningLUTmod7[iMod7]; 

  // ROM for Moduli 17

  generate
  for(i = 0; i < 17; i++)begin: la_WeightROMmod17
      assign weightningLUTmod17[i] = (i*PROD_OF_MODULI/17*14)%PROD_OF_MODULI;
  end: la_WeightROMmod17
  endgenerate

  assign weightedProduct[5] = weightningLUTmod17[iMod17]; 


  // ROM for Moduli 19

  generate
  for(i = 0; i < 19; i++)begin: la_WeightROMmod19
      assign weightningLUTmod19[i] = (i*PROD_OF_MODULI/19*10)%PROD_OF_MODULI;
  end: la_WeightROMmod19
  endgenerate

  assign weightedProduct[6] = weightningLUTmod19[iMod19]; 

  // ROM for Moduli 23

  generate
  for(i = 0; i < 23; i++)begin: la_WeightROMmod23
      assign weightningLUTmod23[i] = (i*PROD_OF_MODULI/23*18)%PROD_OF_MODULI;
  end: la_WeightROMmod23
  endgenerate

  assign weightedProduct[7] = weightningLUTmod23[iMod23]; 


  // ROM for Moduli 29

  generate
  for(i = 0; i < 29; i++)begin: la_WeightROMmod29
      assign weightningLUTmod29[i] = (i*PROD_OF_MODULI/29*2)%PROD_OF_MODULI;
  end: la_WeightROMmod29
  endgenerate

  assign weightedProduct[8] = weightningLUTmod29[iMod29]; 




  // Carry Save Adder Tree
  logic [32:0] AdderTreeStage1s, AdderTreeStage1c;
  logic [32:0] AdderTreeStage2s, AdderTreeStage2c;
  logic [32:0] AdderTreeStage3s, AdderTreeStage3c;
  logic [33:0] AdderTreeStage4s, AdderTreeStage4c;
  logic [33:0] AdderTreeStage5s, AdderTreeStage5c;
  logic [34:0] AdderTreeStage6s, AdderTreeStage6c;
  logic [35:0] AdderTreeStage7s, AdderTreeStage7c;

  CsAdder#(.ADDER_WORD_WIDTH(33)) u_CsAdderTreeStage1(weightedProduct[4], weightedProduct[3], weightedProduct[2], AdderTreeStage1s, AdderTreeStage1c);
  CsAdder#(.ADDER_WORD_WIDTH(33)) u_CsAdderTreeStage2(weightedProduct[5], weightedProduct[1], weightedProduct[0], AdderTreeStage2s, AdderTreeStage2c);
  CsAdder#(.ADDER_WORD_WIDTH(33)) u_CsAdderTreeStage3(weightedProduct[8], weightedProduct[7], weightedProduct[6], AdderTreeStage3s, AdderTreeStage3c);
  CsAdder#(.ADDER_WORD_WIDTH(34)) u_CsAdderTreeStage4({1'b0,AdderTreeStage3s}, {1'b0,AdderTreeStage1s}, {AdderTreeStage3c,1'b0}, AdderTreeStage4s, AdderTreeStage4c);
  CsAdder#(.ADDER_WORD_WIDTH(34)) u_CsAdderTreeStage5({1'b0,AdderTreeStage2s}, {AdderTreeStage1c,1'b0}, {AdderTreeStage2c, 1'b0}, AdderTreeStage5s, AdderTreeStage5c);
  CsAdder#(.ADDER_WORD_WIDTH(35)) u_CsAdderTreeStage6({1'b0,AdderTreeStage4s}, {AdderTreeStage4c,1'b0}, {AdderTreeStage5c, 1'b0}, AdderTreeStage6s, AdderTreeStage6c);
  CsAdder#(.ADDER_WORD_WIDTH(36)) u_CsAdderTreeStage7({1'b0,AdderTreeStage6s}, {AdderTreeStage6c,1'b0}, {1'b0, 1'b0, AdderTreeStage5s}, AdderTreeStage7s, AdderTreeStage7c);


  //Correction Rom 

  logic[36:0] correction;

  assign correction = - 4 * PROD_OF_MODULI; //max correction
  //Correction adder
  
  logic[36:0] corrAdders, corrAdderc, binaryNumber1; 
  logic[35:0] binaryNumber2, binaryNumber3, binaryNumber4;
  
  CsAdder#(.ADDER_WORD_WIDTH(37)) u_CsAdderCorrAdder({1'b0, AdderTreeStage7s},{AdderTreeStage7c, 1'b0} , correction, corrAdders, corrAdderc);
  
  assign binaryNumber1 = corrAdders + (corrAdderc << 1);

  always @(*)
  begin
    if(binaryNumber1[36] == 1'b0)begin
      binaryNumber2 = binaryNumber1 - (2*PROD_OF_MODULI);
    end
    else begin
      binaryNumber2 = binaryNumber1 + (2*PROD_OF_MODULI);
    end
  end
  
  logic [32:0] outPositive;
  always @(*)
  begin
    if(binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b1)
      binaryNumber3 = binaryNumber2 + (PROD_OF_MODULI);
    else if (binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b0)
      binaryNumber3 = binaryNumber2 - (2*PROD_OF_MODULI);
    else if(binaryNumber1[36] == 1'b1 && binaryNumber2[35] == 1'b0)
      binaryNumber3 = binaryNumber2 - (PROD_OF_MODULI);
    else
      binaryNumber3 = binaryNumber2 + (2*PROD_OF_MODULI);
  end

  always @(*)
  begin
    if(binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b0 && binaryNumber3[35] == 1'b0)begin
      outPositive = binaryNumber3;
      binaryNumber4 = 0;
    end
    else if (binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b0 && binaryNumber3[35] == 1'b1)begin
      binaryNumber4 = binaryNumber3 + (PROD_OF_MODULI);
      if (binaryNumber4[35] == 1'b1)
        outPositive = binaryNumber2;
      else
	outPositive = binaryNumber4;
    end
    else if (binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b1 && binaryNumber3[35] == 1'b0)begin
      outPositive = binaryNumber3;
      binaryNumber4 = 0;
    end
    else if (binaryNumber1[36] == 1'b0 && binaryNumber2[35] == 1'b1 && binaryNumber3[35] == 1'b1)begin
      outPositive = binaryNumber1;
      binaryNumber4 = 0;
    end
    else if (binaryNumber1[36] == 1'b1 && binaryNumber2[35] == 1'b0 && binaryNumber3[35] == 1'b0)begin
      outPositive = binaryNumber3;
      binaryNumber4 = 0;
    end
    else if (binaryNumber1[36] == 1'b1 && binaryNumber2[35] == 1'b0 && binaryNumber3[35] == 1'b1)begin
      binaryNumber4 = 0;
      outPositive = binaryNumber2;
    end
    else if (binaryNumber1[36] == 1'b1 && binaryNumber2[35] == 1'b1 && binaryNumber3[35] == 1'b0)begin
      binaryNumber4 = binaryNumber3 - (PROD_OF_MODULI);
      if( binaryNumber4[35] == 1'b1)
	outPositive = binaryNumber3;
      else
        outPositive = binaryNumber4;
    end
    else begin
      binaryNumber4 = 0;
      outPositive = binaryNumber3 + (PROD_OF_MODULI);
    end
  end 

  assign out = outPositive > ((PROD_OF_MODULI)/2) ? outPositive - (PROD_OF_MODULI) : outPositive;
  
endmodule
