//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

  `include "NumberOfTaps.sv"

module Receiver32bits #( 
  parameter INCLUDE_RECEIVER16BITS    = pa_Receiver32bits::INCLUDE_RECEIVER16BITS,
  parameter NUMBER_OF_MODULI	      = pa_Receiver32bits::FIR_NUMBER_OF_MODULI
  //parameter integer MODULI [NUMBER_OF_MODULI - 1:0]      = pa_Receiver24bits::FIR_MODULI 
  )(
  input logic [31:0] signalInI,
  input logic [31:0] signalInQ,
  input logic [1:0] coeffMod3 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod5 [NUMBER_OF_TAPS -1:0],
  input logic [2:0] coeffMod7 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod13 [NUMBER_OF_TAPS -1:0],
  input logic [3:0] coeffMod16 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod17 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod19 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod23 [NUMBER_OF_TAPS -1:0],
  input logic [4:0] coeffMod29 [NUMBER_OF_TAPS -1:0],
  input ck, 
  output logic[31:0] converterOutI,
  output logic[31:0] converterOutQ
  );
  localparam integer MODULI [8:0] = {3,5,7,13,16,17,19,23,29};


  logic [1:0] convOutMod3I, convOutMod3Q; 
  logic [2:0] convOutMod5I, convOutMod5Q; 
  logic [2:0] convOutMod7I, convOutMod7Q;
  logic [3:0] convOutMod13I, convOutMod13Q;
  logic [3:0] convOutMod16I, convOutMod16Q;
  logic [4:0] convOutMod17I, convOutMod17Q;
  logic [4:0] convOutMod19I, convOutMod19Q;
  logic [4:0] convOutMod23I, convOutMod23Q;
  logic [4:0] convOutMod29I, convOutMod29Q;


  logic [1:0] convOutMod3Ireg, convOutMod3Qreg; 
  logic [2:0] convOutMod5Ireg, convOutMod5Qreg; 
  logic [2:0] convOutMod7Ireg, convOutMod7Qreg;
  logic [3:0] convOutMod13Ireg, convOutMod13Qreg;
  logic [3:0] convOutMod16Ireg, convOutMod16Qreg;
  logic [4:0] convOutMod17Ireg, convOutMod17Qreg;
  logic [4:0] convOutMod19Ireg, convOutMod19Qreg;
  logic [4:0] convOutMod23Ireg, convOutMod23Qreg;
  logic [4:0] convOutMod29Ireg, convOutMod29Qreg;

  logic [1:0] filterOutMod3I, filterOutMod3Q; 
  logic [2:0] filterOutMod5I, filterOutMod5Q; 
  logic [2:0] filterOutMod7I, filterOutMod7Q;
  logic [3:0] filterOutMod13I, filterOutMod13Q;
  logic [3:0] filterOutMod16I, filterOutMod16Q;
  logic [4:0] filterOutMod17I, filterOutMod17Q;
  logic [4:0] filterOutMod19I, filterOutMod19Q;
  logic [4:0] filterOutMod23I, filterOutMod23Q;
  logic [4:0] filterOutMod29I, filterOutMod29Q;

  logic [1:0] filterOutMod3Ireg, filterOutMod3Qreg; 
  logic [2:0] filterOutMod5Ireg, filterOutMod5Qreg; 
  logic [2:0] filterOutMod7Ireg, filterOutMod7Qreg;
  logic [3:0] filterOutMod13Ireg, filterOutMod13Qreg;
  logic [3:0] filterOutMod16Ireg, filterOutMod16Qreg;
  logic [4:0] filterOutMod17Ireg, filterOutMod17Qreg;
  logic [4:0] filterOutMod19Ireg, filterOutMod19Qreg;
  logic [4:0] filterOutMod23Ireg, filterOutMod23Qreg;
  logic [4:0] filterOutMod29Ireg, filterOutMod29Qreg;

// Input Converters

InputConverter #(.WORD_LENGTH(32),
  .MODULO(3),
  .MODULO_WORD_LENGTH(2)
  ) u_InputConverterMod3I (
    .in (signalInI),
    .out (convOutMod3I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5I (
    .in (signalInI),
    .out (convOutMod5I)
    );
assign convOutMod16I = signalInI[3:0];
InputConverter #(.WORD_LENGTH(32),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7I (
    .in (signalInI),
    .out (convOutMod7I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(13),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod13I (
    .in (signalInI),
    .out (convOutMod13I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(17),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod17I (
    .in (signalInI),
    .out (convOutMod17I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(19),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod19I (
    .in (signalInI),
    .out (convOutMod19I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(23),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod23I (
    .in (signalInI),
    .out (convOutMod23I)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(29),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod29I (
    .in (signalInI),
    .out (convOutMod29I)
    );

InputConverter #(.WORD_LENGTH(32),
  .MODULO(3),
  .MODULO_WORD_LENGTH(2)
  ) u_InputConverterMod3Q (
    .in (signalInQ),
    .out (convOutMod3Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(5),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod5Q (
    .in (signalInQ),
    .out (convOutMod5Q)
    );
assign convOutMod16Q = signalInQ[3:0];
InputConverter #(.WORD_LENGTH(32),
  .MODULO(7),
  .MODULO_WORD_LENGTH(3)
  ) u_InputConverterMod7Q (
    .in (signalInQ),
    .out (convOutMod7Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(13),
  .MODULO_WORD_LENGTH(4)
  ) u_InputConverterMod13Q (
    .in (signalInQ),
    .out (convOutMod13Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(17),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod17Q (
    .in (signalInQ),
    .out (convOutMod17Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(19),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod19Q (
    .in (signalInQ),
    .out (convOutMod19Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(23),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod23Q (
    .in (signalInQ),
    .out (convOutMod23Q)
    );
InputConverter #(.WORD_LENGTH(32),
  .MODULO(29),
  .MODULO_WORD_LENGTH(5)
  ) u_InputConverterMod29Q (
    .in (signalInQ),
    .out (convOutMod29Q)
    );

always @(posedge ck)begin
  convOutMod3Ireg <= convOutMod3I;
  convOutMod3Qreg <= convOutMod3Q;
  convOutMod5Ireg <= convOutMod5I;
  convOutMod5Qreg <= convOutMod5Q;
  convOutMod7Ireg <= convOutMod7I;
  convOutMod7Qreg <= convOutMod7Q;
  convOutMod13Ireg <= convOutMod13I;
  convOutMod13Qreg <= convOutMod13Q;
  convOutMod16Ireg <= convOutMod16I;
  convOutMod16Qreg <= convOutMod16Q;
  convOutMod17Ireg <= convOutMod17I;
  convOutMod17Qreg <= convOutMod17Q;
  convOutMod19Ireg <= convOutMod19I;
  convOutMod19Qreg <= convOutMod19Q;
  convOutMod23Ireg <= convOutMod23I;
  convOutMod23Qreg <= convOutMod23Q;
  convOutMod29Ireg <= convOutMod29I;
  convOutMod29Qreg <= convOutMod29Q;
end

// FIR Filter  
  
HybridFirLinear #(.NUMBER_OF_MODULI(9), .MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearI(.ck(ck), .iMod3I(convOutMod3Ireg), .iMod5I(convOutMod5Ireg), .iMod7I(convOutMod7Ireg), .iMod13I(convOutMod13Ireg), .iMod16I(convOutMod16Ireg), .iMod17I(convOutMod17Ireg), .iMod19I(convOutMod19Ireg), .iMod23I(convOutMod23Ireg), .iMod29I(convOutMod29Ireg), .coeffMod3(coeffMod3), .coeffMod5(coeffMod5), .coeffMod16(coeffMod16), .coeffMod7(coeffMod7), .coeffMod13(coeffMod13), .coeffMod17(coeffMod17), .coeffMod19(coeffMod19), .coeffMod23(coeffMod23), .coeffMod29(coeffMod29), .oMod3I(filterOutMod3I), .oMod5I(filterOutMod5I), .oMod7I(filterOutMod7I), .oMod13I(filterOutMod13I), .oMod16I(filterOutMod16I), .oMod17I(filterOutMod17I), .oMod19I(filterOutMod19I), .oMod23I(filterOutMod23I), .oMod29I(filterOutMod29I));


HybridFirLinear #(.NUMBER_OF_MODULI(9), .MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS)) u_HybridFirLinearQ(.ck(ck), .iMod3I(convOutMod3Qreg), .iMod5I(convOutMod5Qreg), .iMod7I(convOutMod7Qreg), .iMod13I(convOutMod13Qreg), .iMod16I(convOutMod16Qreg), .iMod17I(convOutMod17Qreg), .iMod19I(convOutMod19Qreg), .iMod23I(convOutMod23Qreg), .iMod29I(convOutMod29Qreg),  .coeffMod3(coeffMod3),  .coeffMod5(coeffMod5), .coeffMod16(coeffMod16), .coeffMod7(coeffMod7), .coeffMod13(coeffMod13), .coeffMod17(coeffMod17), .coeffMod19(coeffMod19), .coeffMod23(coeffMod23), .coeffMod29(coeffMod29), .oMod3I(filterOutMod3Q), .oMod5I(filterOutMod5Q), .oMod7I(filterOutMod7Q), .oMod13I(filterOutMod13Q), .oMod16I(filterOutMod16Q), .oMod17I(filterOutMod17Q), .oMod19I(filterOutMod19Q), .oMod23I(filterOutMod23Q), .oMod29I(filterOutMod29Q));


// Output Converter

always @(posedge ck)begin
  filterOutMod3Ireg <= filterOutMod3I;
  filterOutMod3Qreg <= filterOutMod3Q;
  filterOutMod5Ireg <= filterOutMod5I;
  filterOutMod5Qreg <= filterOutMod5Q;
  filterOutMod7Ireg <= filterOutMod7I;
  filterOutMod7Qreg <= filterOutMod7Q;
  filterOutMod13Ireg <= filterOutMod13I;
  filterOutMod13Qreg <= filterOutMod13Q;
  filterOutMod16Ireg <= filterOutMod16I;
  filterOutMod16Qreg <= filterOutMod16Q;
  filterOutMod17Ireg <= filterOutMod17I;
  filterOutMod17Qreg <= filterOutMod17Q;
  filterOutMod19Ireg <= filterOutMod19I;
  filterOutMod19Qreg <= filterOutMod19Q;
  filterOutMod23Ireg <= filterOutMod23I;
  filterOutMod23Qreg <= filterOutMod23Q;
  filterOutMod29Ireg <= filterOutMod29I;
  filterOutMod29Qreg <= filterOutMod29Q;
end

OutputConverter#(.NUMBER_OF_MODULI(9),.OUTPUT_WORD_LENGTH(32)) u_OutputConverterI(
  .iMod3(filterOutMod3Ireg),
  .iMod5(filterOutMod5Ireg),
  .iMod7(filterOutMod7Ireg),
  .iMod13(filterOutMod13Ireg),
  .iMod16(filterOutMod16Ireg),
  .iMod17(filterOutMod17Ireg),
  .iMod19(filterOutMod19Ireg),
  .iMod23(filterOutMod23Ireg),
  .iMod29(filterOutMod29Ireg),
  .out(converterOutI)
);

OutputConverter#(.NUMBER_OF_MODULI(9),.OUTPUT_WORD_LENGTH(32)) u_OutputConverterQ(
  .iMod3(filterOutMod3Qreg),
  .iMod5(filterOutMod5Qreg),
  .iMod7(filterOutMod7Qreg),
  .iMod13(filterOutMod13Qreg),
  .iMod16(filterOutMod16Qreg),
  .iMod17(filterOutMod17Qreg),
  .iMod19(filterOutMod19Qreg),
  .iMod23(filterOutMod23Qreg),
  .iMod29(filterOutMod29Qreg),
  .out(converterOutQ)
);

endmodule
