//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

module test_Receiver32bits (
  
  );

  localparam integer MODULI [8:0] = {3, 5, 16, 7, 13, 17, 19, 23, 29};

  int   errorCnt; //total error count
  int   errorCntTb; //Error count of tb
  int   errorCntAssertion; //Error count assertion
  logic [31:0] signalInI,signalOutI, signalInQ, signalOutQ;
  logic ck;
  // -- Interfaces and signals
  inTest_Receiver24bits uin_Receiver24bits();
  
  `include "../../rtl/Coefficients.sv"
  `include "../../rtl/NumberOfTaps.sv"
  
  logic [3:0] coeffMod16 [NUMBER_OF_TAPS -1:0];
  logic [1:0] coeffMod3 [NUMBER_OF_TAPS -1:0];
  logic [2:0] coeffMod5 [NUMBER_OF_TAPS -1:0];
  logic [2:0] coeffMod7 [NUMBER_OF_TAPS -1:0];
  logic [3:0] coeffMod13 [NUMBER_OF_TAPS -1:0];
  logic [4:0] coeffMod17 [NUMBER_OF_TAPS -1:0];
  logic [4:0] coeffMod19 [NUMBER_OF_TAPS -1:0];
  logic [4:0] coeffMod23 [NUMBER_OF_TAPS -1:0];
  logic [4:0] coeffMod29 [NUMBER_OF_TAPS -1:0];

  assign coeffMod3 = $unsigned(COEFFICIENTS3[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod5 = $unsigned(COEFFICIENTS5[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod16 = $unsigned(COEFFICIENTS16[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod7 = $unsigned(COEFFICIENTS7[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod13 = $unsigned(COEFFICIENTS13[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod17 = $unsigned(COEFFICIENTS17[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod19 = $unsigned(COEFFICIENTS19[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod23 = $unsigned(COEFFICIENTS23[NUMBER_OF_TAPS - 1:0]);
  assign coeffMod29 = $unsigned(COEFFICIENTS29[NUMBER_OF_TAPS - 1:0]);

//  Receiver32bits #(.NUMBER_OF_MODULI(8)) u_Receiver32bits(signalInI, signalInQ, coeffMod3, coeffMod5, coeffMod7, coeffMod13, coeffMod16, coeffMod17, coeffMod19, coeffMod23, coeffMod29, ck, signalOutI, signalOutQ);
  Receiver32bits u_Receiver32bits(signalInI, signalInQ, (2*NUMBER_OF_TAPS)'(coeffMod3),(3*NUMBER_OF_TAPS)'(coeffMod5),(3*NUMBER_OF_TAPS)'(coeffMod7),(4*NUMBER_OF_TAPS)'(coeffMod13),(4*NUMBER_OF_TAPS)'(coeffMod16),(5*NUMBER_OF_TAPS)'(coeffMod17),(5*NUMBER_OF_TAPS)'(coeffMod19),(5*NUMBER_OF_TAPS)'(coeffMod23),(5*NUMBER_OF_TAPS)'(coeffMod29), ck, signalOutI, signalOutQ);

  // ------------------------------
  // -- Clock and Reset  
  // ------------------------------
  
  initial begin
    uin_Receiver24bits.arst = 0;
  end
  
  initial begin
    uin_Receiver24bits.clockEnable = 0;
  end
  
  //int cnt;
//-------------------------------------------------------------------------------------------------
//                Output simulation format/results
//-------------------------------------------------------------------------------------------------
  initial
  begin
    $timeformat(-6, 3, "us", 0);
  end   


  // ------------------------------
  // -- DUT and assignments  
  // ------------------------------
  

  logic [31:0] reffQ, reffI;
  assertionsReceiver24bits u_assertionsReceiver24bits(signalOutI, signalOutQ, reffI, reffQ, ck, errorCnt);
  // ------------------------------
  // -- Concurrent assertions
  // ------------------------------
  
  //-- Clocks and reset
  
  // ------------------------------
  // -- Test program  
  // ------------------------------
  //-------------------------------------------------------------------------------------------------
//                Main Stimulus
//-------------------------------------------------------------------------------------------------

logic [31:0] stimuliI [999:0];
logic [31:0] stimuliQ [999:0];
logic [31:0] corrI [1000:0];
logic [31:0] corrQ [1000:0];
`include "signal.v"
`include "results.v"

int i = 0;
initial begin
  @(posedge ck)
  for(i = 1;i <4096; i++) begin
    @(posedge ck);
  end

  fu_printEndStatus();
  $stop;
end


initial begin 
  ck <= 0;
  while(1) begin
    #50;
    ck = ~ck;
  end
end


assign signalInI = stimuliI[i];
assign signalInQ = stimuliQ[i];
assign reffQ = corrQ[i+2];
assign reffI = corrI[i+2];

function void fu_printEndStatus;
    $display("");
    $display("");
    $display($time, " ns:");
    $display("--------------------------------------------------------------------------------");
    $display("--------------------------------------------------------------------------------");
    $display("");
    $display("  SIMULATIONS FINISHED WITH %0d ERROR(S).", errorCnt);
    $display("");
    $display("");
    $display("      -> Check functional coverage.");
    $display("");
    $display("--------------------------------------------------------------------------------");
    $display("--------------------------------------------------------------------------------");
    $display("");
    $display("");
endfunction    

endmodule
