//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 03-24-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

  `include "NumberOfTaps.sv"

module Receiver32bits #( 
  parameter INCLUDE_RECEIVER16BITS    = pa_Receiver32bits::INCLUDE_RECEIVER16BITS,
  parameter NUMBER_OF_MODULI	      = pa_Receiver32bits::FIR_NUMBER_OF_MODULI,
  parameter integer MODULI [NUMBER_OF_MODULI - 1:0]      = pa_Receiver32bits::FIR_MODULI 
  )(
  input logic [15:0] signalInI,
  input logic [15:0] signalInQ,
  input logic [15:0] coefficients [NUMBER_OF_TAPS - 1:0], 
  input ck, 
  output logic[31:0] converterOutI,
  output logic[31:0] converterOutQ
  );
  
  
  logic [15:0] regX_I [NUMBER_OF_TAPS-1 : 0];
  logic [15:0] regXnext_I [NUMBER_OF_TAPS-1 : 0];
  logic [31:0] regMult_I [NUMBER_OF_TAPS-1 : 0];
  logic [31:0] regY_I [NUMBER_OF_TAPS-1 : 0];
  logic [15:0] regX_Q [NUMBER_OF_TAPS-1 : 0];
  logic [15:0] regXnext_Q [NUMBER_OF_TAPS-1 : 0];
  logic [31:0] regMult_Q [NUMBER_OF_TAPS-1 : 0];
  logic [31:0] regY_Q [NUMBER_OF_TAPS-1 : 0];
  
  genvar i;
  generate 
  for( i = 0; i< NUMBER_OF_TAPS; i++) begin: la_FIR
    if (i == 0) begin
      assign regY_I[i] = $signed(regX_I[i]) * $signed(coefficients[i]);
      assign regY_Q[i] = $signed(regX_Q[i]) * $signed(coefficients[i]);
    end
    else begin
      assign regMult_I[i] = $signed(regX_I[i]) * $signed(coefficients[i]);
      assign regMult_Q[i] = $signed(regX_Q[i]) * $signed(coefficients[i]);
      assign regY_I[i] = regMult_I[i] + regY_I[i-1];
      assign regY_Q[i] = regMult_Q[i] + regY_Q[i-1]; 
    end
  end:la_FIR
  endgenerate

  generate 
  for( i = 1; i< NUMBER_OF_TAPS; i++) begin: la_RegNext
    assign regX_I[i] = regXnext_I[i];
    assign regX_Q[i] = regXnext_Q[i];
  end:la_RegNext
  endgenerate

  always @(posedge ck) begin
    for(int i = 0; i < NUMBER_OF_TAPS - 1; i++) begin
      regXnext_I[i+1] = regX_I[i];
      regXnext_Q[i+1] = regX_Q[i];
    end
  end
  
    

  assign regX_I[0] = signalInI;
  assign regX_Q[0] = signalInQ;

  logic [31:0] regOutI,regOutQ;

  always @(posedge ck) begin
    regOutI = regY_I[NUMBER_OF_TAPS -1];
    regOutQ = regY_Q[NUMBER_OF_TAPS -1];
  end

  assign converterOutI = regOutI;
  assign converterOutQ = regOutQ;
  
endmodule
