//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 04-07-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

  `include "NumberOfTaps.sv"
module ReceiverQuad  (
  input logic [11:0] signalInI,
  input logic [11:0] signalInQ,
  logic [3:0] coeffMod13I [NUMBER_OF_TAPS - 1:0],
  logic [4:0] coeffMod17I [NUMBER_OF_TAPS - 1:0],
  logic [5:0] coeffMod37I [NUMBER_OF_TAPS - 1:0], 
  logic [3:0] coeffMod13Q [NUMBER_OF_TAPS - 1:0],
  logic [4:0] coeffMod17Q [NUMBER_OF_TAPS - 1:0],
  logic [5:0] coeffMod37Q [NUMBER_OF_TAPS - 1:0], 
  input logic ck,
  output logic [11:0] signalOutI,
  output logic [11:0] signalOutQ
  );
  
  localparam integer MODULI[2:0] = {13,17,37};

  logic [3:0] signalInMod13I;
  logic [4:0] signalInMod17I;
  logic [5:0] signalInMod37I;

  logic [3:0] signalInMod13Q;
  logic [4:0] signalInMod17Q;
  logic [5:0] signalInMod37Q;

  
  logic [3:0] signalInMod13N;
  logic [4:0] signalInMod17N;
  logic [5:0] signalInMod37N;

  logic [3:0] signalInMod13T;
  logic [4:0] signalInMod17T;
  logic [5:0] signalInMod37T;

  logic [3:0] signalInMod13Nreg;
  logic [4:0] signalInMod17Nreg;
  logic [5:0] signalInMod37Nreg;

  logic [3:0] signalInMod13Treg;
  logic [4:0] signalInMod17Treg;
  logic [5:0] signalInMod37Treg;

  logic [3:0] filterOutMod13N;
  logic [4:0] filterOutMod17N;
  logic [5:0] filterOutMod37N;

  logic [3:0] filterOutMod13T;
  logic [4:0] filterOutMod17T;
  logic [5:0] filterOutMod37T;
  
  logic [3:0] filterOutMod13I;
  logic [4:0] filterOutMod17I;
  logic [5:0] filterOutMod37I;

  logic [3:0] filterOutMod13Q;
  logic [4:0] filterOutMod17Q;
  logic [5:0] filterOutMod37Q;

  logic [3:0] filterOutMod13Treg;
  logic [4:0] filterOutMod17Treg;
  logic [5:0] filterOutMod37Treg;

  logic [3:0] filterOutMod13Nreg;
  logic [4:0] filterOutMod17Nreg;
  logic [5:0] filterOutMod37Nreg;

  // Input Converters

  InputConverter#(.WORD_LENGTH(12),.MODULO(13),.MODULO_WORD_LENGTH(4)) u_InputConverterIMod13(signalInI, signalInMod13I);
  InputConverter#(.WORD_LENGTH(12),.MODULO(13),.MODULO_WORD_LENGTH(4)) u_InputConverterQMod13(signalInQ, signalInMod13Q);

  InputConverter#(.WORD_LENGTH(12),.MODULO(17),.MODULO_WORD_LENGTH(5)) u_InputConverterIMod17(signalInI, signalInMod17I);
  InputConverter#(.WORD_LENGTH(12),.MODULO(17),.MODULO_WORD_LENGTH(5)) u_InputConverterQMod17(signalInQ, signalInMod17Q);

  InputConverter#(.WORD_LENGTH(12),.MODULO(37),.MODULO_WORD_LENGTH(6)) u_InputConverterIMod37(signalInI, signalInMod37I);
  InputConverter#(.WORD_LENGTH(12),.MODULO(37),.MODULO_WORD_LENGTH(6)) u_InputConverterQMod37(signalInQ, signalInMod37Q);

  // RNS to QRNS Converter

  Rns2QrnsConverter#(.WORD_LENGTH(4),.MODULO(13),.J(8),.J_WORD_LENGTH(4)) u_Rns2QrnsConverterMod13(signalInMod13I,signalInMod13Q, signalInMod13N, signalInMod13T);
  Rns2QrnsConverter#(.WORD_LENGTH(5),.MODULO(17),.J(4),.J_WORD_LENGTH(3)) u_Rns2QrnsConverterMod17(signalInMod17I,signalInMod17Q, signalInMod17N, signalInMod17T);
  Rns2QrnsConverter#(.WORD_LENGTH(6),.MODULO(37),.J(6),.J_WORD_LENGTH(3)) u_Rns2QrnsConverterMod37(signalInMod37I,signalInMod37Q, signalInMod37N, signalInMod37T);

always @(posedge ck) begin
  signalInMod13Nreg <= signalInMod13N;
  signalInMod13Treg <= signalInMod13T;
  signalInMod17Nreg <= signalInMod17N;
  signalInMod17Treg <= signalInMod17T;
  signalInMod37Nreg <= signalInMod37N;
  signalInMod37Treg <= signalInMod37T;
end
  
  // FIR Filter
  
  HybridFirLinear#(.NUMBER_OF_MODULI(3),.MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS),.CONJUGATE(0)) u_HybridFirLinearN(ck,signalInMod13Nreg,signalInMod17Nreg,signalInMod37Nreg,coeffMod13I,coeffMod17I,coeffMod37I,filterOutMod13N,filterOutMod17N,filterOutMod37N);

  HybridFirLinear#(.NUMBER_OF_MODULI(3),.MODULI(MODULI), .NUMBER_OF_TAPS(NUMBER_OF_TAPS),.CONJUGATE(1)) u_HybridFirLinearT(ck,signalInMod13Treg,signalInMod17Treg,signalInMod37Treg,coeffMod13Q,coeffMod17Q,coeffMod37Q,filterOutMod13T,filterOutMod17T,filterOutMod37T);

always @(posedge ck) begin
  filterOutMod13Nreg <= filterOutMod13N;
  filterOutMod13Treg <= filterOutMod13T;
  filterOutMod17Nreg <= filterOutMod17N;
  filterOutMod17Treg <= filterOutMod17T;
  filterOutMod37Nreg <= filterOutMod37N;
  filterOutMod37Treg <= filterOutMod37T;
end

  // QRNS to RNS Converter

  Qrns2RnsConverter#(.WORD_LENGTH(4),.MODULO(13),.MULT_INV_J(5),.MULT_INV_2(7)) u_Qrns2RnsConverterMod13(filterOutMod13Nreg,filterOutMod13Treg,filterOutMod13I,filterOutMod13Q);
  
  Qrns2RnsConverter#(.WORD_LENGTH(5),.MODULO(17),.MULT_INV_J(13),.MULT_INV_2(9)) u_Qrns2RnsConverterMod17(filterOutMod17Nreg,filterOutMod17Treg,filterOutMod17I,filterOutMod17Q);

  Qrns2RnsConverter#(.WORD_LENGTH(6),.MODULO(37),.MULT_INV_J(31),.MULT_INV_2(19)) u_Qrns2RnsConverterMod37(filterOutMod37Nreg,filterOutMod37Treg,filterOutMod37I,filterOutMod37Q);

  // Output Converter
  OutputConverter#(.OUTPUT_WORD_LENGTH(12),.NUMBER_OF_MODULI(3)) u_OutputCOnverterI(filterOutMod13I,filterOutMod17I,filterOutMod37I,signalOutI);
  OutputConverter#(.OUTPUT_WORD_LENGTH(12),.NUMBER_OF_MODULI(3)) u_OutputCOnverterQ(filterOutMod13Q,filterOutMod17Q,filterOutMod37Q,signalOutQ);
  
endmodule

