//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module HybridFirLinear #(
  parameter NUMBER_OF_MODULI								= pa_ReceiverQuad::FIR_NUMBER_OF_MODULI,
  parameter integer MODULI[NUMBER_OF_MODULI -1 :0]					= pa_ReceiverQuad::FIR_MODULI,
  parameter NUMBER_OF_TAPS								= pa_ReceiverQuad::FIR_NUMBER_OF_TAPS,
  parameter CONJUGATE									= pa_ReceiverQuad::FIR_CONJUGATE
  )(
  input logic ck,
  input logic [3:0] iMod13I,
  input logic [4:0] iMod17I,
  input logic [5:0] iMod37I,
  input logic [3:0] coeffMod13I [NUMBER_OF_TAPS - 1:0],
  input logic [4:0] coeffMod17I [NUMBER_OF_TAPS - 1:0],
  input logic [5:0] coeffMod37I [NUMBER_OF_TAPS - 1:0],
  output logic [3:0] oMod13I,
  output logic [4:0] oMod17I,
  output logic [5:0] oMod37I
  );

// MODULO 13 FIR filter 

`include "CoefficientsQuad.sv"

 


logic [NUMBER_OF_TAPS:0][3:0] xMod13I;
logic [3:0] xMod13Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][3:0] xMod13Ireg;
logic [NUMBER_OF_TAPS:0][8:0] yMod13I;
genvar i;
generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod13
    HybridFirCellLinear#(.WORD_LENGTH(4),.MODULO(13)) u_HybridCellMod13(xMod13I[i],yMod13I[i],coeffMod13I[i],xMod13Ireg[i+1],yMod13I[i+1]);
    assign xMod13I[i+1] = xMod13Inext[i+1];
end: la_firMod13
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod13
    xMod13Inext[i] <= xMod13Ireg[i];
  end: la_regMod13
end

assign xMod13I[0] = iMod13I;
assign yMod13I[0] = 9'b000000000;

InputConverter #(.WORD_LENGTH(10),.MODULO(13),.MODULO_WORD_LENGTH(4)) u_ResidueConverterMod13I({1'b0,yMod13I[NUMBER_OF_TAPS]},oMod13I);

// MODULO 17 FIR filter 

logic [NUMBER_OF_TAPS:0][4:0] xMod17I;
logic [NUMBER_OF_TAPS:0][4:0] xMod17Ireg;
logic [4:0] xMod17Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][10:0] yMod17I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod17
    HybridFirCellLinear#(.WORD_LENGTH(5),.MODULO(17)) u_HybridCellMod17(xMod17I[i],yMod17I[i],coeffMod17I[i],xMod17Ireg[i+1],yMod17I[i+1]);
    assign xMod17I[i+1] = xMod17Inext[i+1];
end: la_firMod17
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod17
    xMod17Inext[i] <= xMod17Ireg[i];
  end: la_regMod17
end

assign xMod17I[0] = iMod17I;
assign yMod17I[0] = 11'b00000000000;
  
InputConverter #(.WORD_LENGTH(12),.MODULO(17),.MODULO_WORD_LENGTH(5)) u_ResidueConverterMod17I({1'b0,yMod17I[NUMBER_OF_TAPS]},oMod17I);

// MODULO 37 FIR filter 

logic [NUMBER_OF_TAPS:0][5:0] xMod37I;
logic [5:0] xMod37Inext[NUMBER_OF_TAPS:0];
logic [NUMBER_OF_TAPS:0][5:0] xMod37Ireg;
logic [NUMBER_OF_TAPS:0][12:0] yMod37I;

generate 
  for(i = 0; i < NUMBER_OF_TAPS; i++) begin: la_firMod37
    HybridFirCellLinear#(.WORD_LENGTH(6),.MODULO(37)) u_HybridCellMod37(xMod37I[i],yMod37I[i], coeffMod37I[i],xMod37Ireg[i+1],yMod37I[i+1]);
    assign xMod37I[i+1] = xMod37Inext[i+1];
end: la_firMod37
endgenerate

always @(posedge ck) begin
  for(int i = 1; i <= NUMBER_OF_TAPS; i++) begin: la_regMod37
    xMod37Inext[i] <= xMod37Ireg[i];
  end: la_regMod37
end

assign xMod37I[0] = iMod37I;
assign yMod37I[0] = 13'b0000000000000;

InputConverter #(.WORD_LENGTH(14),.MODULO(37),.MODULO_WORD_LENGTH(6)) u_ResidueConverterMod37I({1'b0,yMod37I[NUMBER_OF_TAPS]},oMod37I);
endmodule
