//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module InputConverter #(
  parameter WORD_LENGTH		= pa_ReceiverQuad::OUTPUT_CONVERTER_WORD_LENGTH,
  parameter MODULO		= pa_ReceiverQuad::OUTPUT_CONVERTER_MODULO,
  parameter MODULO_WORD_LENGTH	= pa_ReceiverQuad::MODULO_WORD_LENGTH
  )(
  input logic [WORD_LENGTH - 1:0] in,
  output logic [MODULO_WORD_LENGTH - 1:0] out
  );
  logic[MODULO_WORD_LENGTH - 1:0] Fx[WORD_LENGTH - 1:0];
  logic[MODULO_WORD_LENGTH:0] afterSum[WORD_LENGTH - 1:0];
  logic[MODULO_WORD_LENGTH:0] afterCorrection[WORD_LENGTH - 1:0];
  
  
  genvar i;
  generate
    for(i = 0; i < WORD_LENGTH; i++)begin: la_function
      if( i < WORD_LENGTH - 1) begin 
        assign Fx[i] = 2**i % MODULO;
      end
      else begin
	assign Fx[i] = MODULO - (2**i % MODULO);
      end
    end: la_function
  endgenerate
  
  generate
    for(i = 1; i < WORD_LENGTH; i++)begin: la_add
      assign afterSum[i] = in[i]?  Fx[i] + afterCorrection[i-1] : afterCorrection[i-1];
    end: la_add
  endgenerate

  generate
    for(i = 1; i < WORD_LENGTH; i++)begin: la_corr
      assign afterCorrection[i] = afterSum[i] >= MODULO? afterSum[i] - MODULO : afterSum[i];
    end: la_corr
  endgenerate

  assign afterCorrection[0] = in[0]? Fx[0]: 0;
  assign out = afterCorrection[WORD_LENGTH - 1];
endmodule
