//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module OutputConverter #(
  parameter NUMBER_OF_MODULI			    = pa_ReceiverQuad::OUTPUT_CONVERTER_NUMBER_OF_MODULI,
  parameter OUTPUT_WORD_LENGTH			    = pa_ReceiverQuad::OUTPUT_CONVERTER_WORD_LENGTH
  )(
  input logic [3:0] iMod13,
  input logic [4:0] iMod17,
  input logic [5:0] iMod37,
  output logic [11:0] out
  );
  localparam PROD_OF_MODULI = 13*17*37;
  logic [12 : 0] weightedProduct [NUMBER_OF_MODULI - 1: 0];
  logic [12  : 0] weightningLUTmod13 [12: 0];
  logic [12  : 0] weightningLUTmod17 [16: 0];
  logic [12  : 0] weightningLUTmod37 [36: 0];
  

  // ROM for Moduli 13
  genvar i;
  generate
  for(i = 0; i < 13; i++)begin: la_WeightROMmod13
      assign weightningLUTmod13[i] = (i*17*37*8)%PROD_OF_MODULI;
  end: la_WeightROMmod13
  endgenerate
  
  assign weightedProduct[0] = weightningLUTmod13[iMod13]; 

  // ROM for Moduli 17

  generate
  for(i = 0; i < 17; i++)begin: la_WeightROMmod17
      assign weightningLUTmod17[i] = (i*13*37*7)%PROD_OF_MODULI;
  end: la_WeightROMmod17
  endgenerate

  assign weightedProduct[1] = weightningLUTmod17[iMod17]; 

  // ROM for Moduli 37

  generate
  for(i = 0; i < 37; i++)begin: la_WeightROMmod37
      assign weightningLUTmod37[i] = (i*13*17*36)%PROD_OF_MODULI;
  end: la_WeightROMmod37
  endgenerate

  assign weightedProduct[2] = weightningLUTmod37[iMod37]; 

  // Carry Save Adder Tree
  logic [12:0] AdderTreeStage2s, AdderTreeStage2c;

  CsAdder#(.ADDER_WORD_WIDTH(13)) u_CsAdderTreeStage2(weightedProduct[2], weightedProduct[1], weightedProduct[0], AdderTreeStage2s, AdderTreeStage2c);


  //Correction Rom 

  logic[14:0] correction;

  assign correction =  -13*17*37; //max correction
  //Correction adder
  
  logic[14:0] corrAdders, corrAdderc, binaryNumber1; 
  logic[13:0] binaryNumber2, binaryNumber3;
  
  CsAdder#(.ADDER_WORD_WIDTH(15)) u_CsAdderCorrAdder({1'b0,1'b0, AdderTreeStage2s},{1'b0,AdderTreeStage2c, 1'b0} , correction, corrAdders, corrAdderc);
  
  assign binaryNumber1 = corrAdders + (corrAdderc << 1);

  always @(*)
  begin
    if(binaryNumber1[14] == 1'b0)begin
      binaryNumber2 = binaryNumber1 - (13*17*37);
    end
    else begin
      binaryNumber2 = binaryNumber1 + (13*17*37);
    end
  end
  
  logic [12:0] outPositive;
  always @(*)
  begin
    if(binaryNumber1[14] == 1'b0 && binaryNumber2[13] == 1'b1)
      outPositive = binaryNumber1;
    else 
      outPositive = binaryNumber2;
  end
  
  assign out = outPositive > ((13*17*37)/2) ? outPositive - (13*17*37) : outPositive;
  
endmodule
