//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 02-04-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  


module Qrns2RnsConverter #(
  parameter WORD_LENGTH		= pa_ReceiverQuad::FIR_CELL_WORD_LENGTH,
  parameter MODULO		= pa_ReceiverQuad::FIR_CELL_MODULO,
  parameter MULT_INV_J		= pa_ReceiverQuad::FIR_MULT_INV_J,
  parameter MULT_INV_2		= pa_ReceiverQuad::FIR_MULT_INV_2
  )(
  input logic [WORD_LENGTH - 1:0] inN,
  input logic [WORD_LENGTH - 1:0] inT,
  output logic [WORD_LENGTH - 1:0] outI,
  output logic [WORD_LENGTH - 1:0] outQ
  );
  
  logic [WORD_LENGTH: 0] sum, sub;
  logic [WORD_LENGTH-1: 0] sumMod, subMod;
  logic [WORD_LENGTH-1: 0] IRom[MODULO - 1: 0];
  logic [WORD_LENGTH-1: 0] QRom[MODULO - 1: 0];
  assign sum = inN + inT;
  assign sub = inN - inT;
  
  assign sumMod = sum >= MODULO ? sum - MODULO: sum;
  assign subMod = sub[WORD_LENGTH] == 1'b1 ? sub + MODULO: sub;

  genvar i;
  generate
  for(i= 0; i< MODULO; i++)begin: la_IRom
    assign IRom[i] = (MULT_INV_2 * i)%MODULO;
  end: la_IRom
  endgenerate
    
  
  generate
  for(i= 0; i< MODULO; i++)begin: la_QRom
    assign QRom[i] = (MULT_INV_J*MULT_INV_2 * i)%MODULO;
  end: la_QRom
  endgenerate

  assign outI = IRom[sumMod];
  assign outQ = QRom[subMod];

endmodule
