//====================================================================  
//        Copyright (c) 2015 Nordic Semiconductor ASA, Norway           
//====================================================================  
// Created       : eify at 04-07-16                                 
// Modified      : $Author$ $Date$                                      
// Version       : $Revision$ $HeadURL$                                 
//====================================================================  

  `include "NumberOfTaps.sv"
module ReceiverQuad  (
  input logic [11:0] signalInI,
  input logic [11:0] signalInQ,
  input logic [7:0] coefficients [NUMBER_OF_TAPS-1 : 0],
  input logic ck,
  output logic [11:0] signalOutI,
  output logic [11:0] signalOutQ
  );
  
  logic [5:0] regX_I [NUMBER_OF_TAPS-1 : 0];
  logic [5:0] regXnext_I [NUMBER_OF_TAPS-1 : 0];
  logic [11:0] regMult_I [NUMBER_OF_TAPS-1 : 0];
  logic [11:0] regY_I [NUMBER_OF_TAPS-1 : 0];
  logic [5:0] regX_Q [NUMBER_OF_TAPS-1 : 0];
  logic [5:0] regXnext_Q [NUMBER_OF_TAPS-1 : 0];
  logic [11:0] regMult_Q [NUMBER_OF_TAPS-1 : 0];
  logic [11:0] regY_Q [NUMBER_OF_TAPS-1 : 0];
  
  genvar i;
  generate 
  for( i = 0; i< NUMBER_OF_TAPS; i++) begin: la_FIR
    if (i == 0) begin
      assign regY_I[i] = $signed(regX_I[i]) * $signed(coefficients[i]) - $signed(regX_Q[i]) * $signed(coefficients[i]);
      assign regY_Q[i] = $signed(regX_Q[i]) * $signed(coefficients[i]) + $signed(regX_I[i]) * $signed(coefficients[i]);
    end
    else begin
      assign regMult_I[i] = $signed(regX_I[i]) * $signed(coefficients[i])- $signed(regX_Q[i]) * $signed(coefficients[i]);
      assign regMult_Q[i] = $signed(regX_Q[i]) * $signed(coefficients[i]) + $signed(regX_I[i]) * $signed(coefficients[i]);
      assign regY_I[i] = regMult_I[i] + regY_I[i-1];
      assign regY_Q[i] = regMult_Q[i] + regY_Q[i-1]; 
    end
  end:la_FIR
  endgenerate

  generate 
  for( i = 1; i< NUMBER_OF_TAPS; i++) begin: la_RegNext
    assign regX_I[i] = regXnext_I[i];
    assign regX_Q[i] = regXnext_Q[i];
  end:la_RegNext
  endgenerate

  always @(posedge ck) begin
    for(int i = 0; i < NUMBER_OF_TAPS - 1; i++) begin
      regXnext_I[i+1] = regX_I[i];
      regXnext_Q[i+1] = regX_Q[i];
    end
  end
  
    

  assign regX_I[0] = signalInI;
  assign regX_Q[0] = signalInQ;

  logic [15:0] regOutI,regOutQ;

  always @(posedge ck) begin
    regOutI = regY_I[NUMBER_OF_TAPS -1];
    regOutQ = regY_Q[NUMBER_OF_TAPS -1];
  end

  assign signalOutI = regOutI;
  assign signalOutQ = regOutQ;
 
   
endmodule

